@@ -15,7 +15,7 @@
  ********************************************************************/
 
 /*
- *  RCS $Id: clustal-omega.c 268 2011-11-16 02:34:34Z andreas $
+ *  RCS $Id: clustal-omega.c 290 2013-09-20 15:18:12Z fabian $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -191,14 +191,20 @@ SetDefaultAlnOpts(opts_t *prOpts) {
     
     prOpts->pcDistmatInfile = NULL;
     prOpts->pcDistmatOutfile = NULL;
-	
+
+    prOpts->iClustersizes = 100; /* FS, r274 -> */
+    prOpts->iTransitivity = 0; /* FS, r290 -> */
+    prOpts->pcClustfile = NULL; /* FS, r274 -> */
+    prOpts->pcPosteriorfile = NULL; /* FS, r288 -> */
     prOpts->iClusteringType = CLUSTERING_UPGMA;
     prOpts->iPairDistType = PAIRDIST_KTUPLE;
     prOpts->bUseMbed = TRUE; /* FS, r250 -> */
     prOpts->bUseMbedForIteration = TRUE; /* FS, r250 -> */
+    prOpts->bPileup = FALSE; /* FS, r288 -> */
     prOpts->pcGuidetreeOutfile = NULL;
     prOpts->pcGuidetreeInfile = NULL;
-	
+    prOpts->bPercID = FALSE;
+
     prOpts->ppcHMMInput = NULL;
     prOpts->iHMMInputFiles = 0;
 		
@@ -286,6 +292,7 @@ PrintAlnOpts(FILE *prFile, opts_t *prOpts)
 	fprintf(prFile, "option: pair-dist-type = %d\n", prOpts->iPairDistType);
 	fprintf(prFile, "option: use-mbed = %d\n", prOpts->bUseMbed);
 	fprintf(prFile, "option: use-mbed-for-iteration = %d\n", prOpts->bUseMbedForIteration);
+	fprintf(prFile, "option: pile-up = %d\n", prOpts->bPileup);
 	fprintf(prFile, "option: guidetree-outfile = %s\n", 
             NULL != prOpts->pcGuidetreeOutfile? prOpts->pcGuidetreeOutfile: "(null)");
 	fprintf(prFile, "option: guidetree-infile = %s\n",
@@ -299,6 +306,10 @@ PrintAlnOpts(FILE *prFile, opts_t *prOpts)
 	fprintf(prFile, "option: max-hmm-iterations = %d\n", prOpts->iMaxHMMIterations);
 	fprintf(prFile, "option: max-guidetree-iterations = %d\n", prOpts->iMaxGuidetreeIterations);
     fprintf(prFile, "option: iMacRamMB = %d\n", prOpts->rHhalignPara.iMacRamMB);
+    fprintf(prFile, "option: percent-id = %d\n", prOpts->bPercID);
+    fprintf(prFile, "option: use-kimura = %d\n", prOpts->bUseKimura);
+    fprintf(prFile, "option: clustering-out = %s\n", prOpts->pcClustfile);
+    fprintf(prFile, "option: posterior-out = %s\n", prOpts->pcPosteriorfile);
 
 }
 /* end of PrintAlnOpts() */
@@ -383,7 +394,8 @@ AlnToHHMFile(mseq_t *prMSeq, char *pcHMMOut)
         retcode = FAILURE;
         goto cleanup_and_return;
     }
-    if (WriteAlignment(prMSeq, tmp_aln, MSAFILE_A2M)) {
+#define LINE_WRAP 60
+    if (WriteAlignment(prMSeq, tmp_aln, MSAFILE_A2M, LINE_WRAP, FALSE)) {
         Log(&rLog, LOG_ERROR, "Could not save alignment to %s", tmp_aln);
         retcode = FAILURE;
         goto cleanup_and_return;
@@ -468,7 +480,8 @@ AlnToHMMFile(mseq_t *prMSeq, const char *pcHMMOut)
         retcode = FAILURE;
         goto cleanup_and_return;
     }
-    if (WriteAlignment(prMSeq, tmp_aln, MSAFILE_STOCKHOLM)) {
+#define LINE_WRAP 60
+    if (WriteAlignment(prMSeq, tmp_aln, MSAFILE_STOCKHOLM, LINE_WRAP, FALSE)) {
         Log(&rLog, LOG_ERROR, "Could not save alignment to %s", tmp_aln);
         retcode = FAILURE;
         goto cleanup_and_return;
@@ -740,8 +753,9 @@ SequentialAlignmentOrder(int **piOrderLR_p, int iNumSeq)
 int
 AlignmentOrder(int **piOrderLR_p, double **pdSeqWeights_p, mseq_t *prMSeq,
                int iPairDistType, char *pcDistmatInfile, char *pcDistmatOutfile,
-               int iClusteringType, char *pcGuidetreeInfile, char *pcGuidetreeOutfile,
-               bool bUseMbed)
+               int iClusteringType, int iClustersizes, 
+               char *pcGuidetreeInfile, char *pcGuidetreeOutfile, char *pcClusterFile, 
+               bool bUseMbed, bool bPercID)
 {
     /* pairwise distance matrix (tmat in 1.83) */
     symmatrix_t *distmat = NULL;
@@ -757,7 +771,11 @@ AlignmentOrder(int **piOrderLR_p, double **pdSeqWeights_p, mseq_t *prMSeq,
     if (2==prMSeq->nseqs) {
         Log(&rLog, LOG_VERBOSE,
             "Have only two sequences: No need to compute pairwise score and compute a tree.");
-        
+
+        if (NULL != pcDistmatOutfile){
+            Log(&rLog, LOG_WARN, "Have only two sequences: Will not calculate/print distance matrix.");
+        }
+
         (*piOrderLR_p) = (int*) CKMALLOC(DIFF_NODE * 3 * sizeof(int));
         (*piOrderLR_p)[DIFF_NODE*0+LEFT_NODE] = 0;
         (*piOrderLR_p)[DIFF_NODE*0+RGHT_NODE] = 0;
@@ -798,7 +816,11 @@ AlignmentOrder(int **piOrderLR_p, double **pdSeqWeights_p, mseq_t *prMSeq,
     } else {
 
         if (bUseMbed) {
-            if (Mbed(&prTree, prMSeq, iPairDistType, pcGuidetreeOutfile)) {
+            if (NULL != pcDistmatInfile) {
+                Log(&rLog, LOG_ERROR, "Can't input distance matrix when in mbed mode.");
+                return FAILURE;                
+            }
+            if (Mbed(&prTree, prMSeq, iPairDistType, pcGuidetreeOutfile, iClustersizes, pcClusterFile)) {
                 Log(&rLog, LOG_ERROR, "mbed execution failed.");
                 return FAILURE;
             }
@@ -809,7 +831,7 @@ AlignmentOrder(int **piOrderLR_p, double **pdSeqWeights_p, mseq_t *prMSeq,
             }
         } else {
 
-            if (PairDistances(&distmat, prMSeq, iPairDistType,
+            if (PairDistances(&distmat, prMSeq, iPairDistType, bPercID, 
                               0, prMSeq->nseqs, 0, prMSeq->nseqs,
                               pcDistmatInfile, pcDistmatOutfile)) {
                 Log(&rLog, LOG_ERROR, "Couldn't compute pair distances");
@@ -833,8 +855,9 @@ AlignmentOrder(int **piOrderLR_p, double **pdSeqWeights_p, mseq_t *prMSeq,
                 Log(&rLog, LOG_FATAL, "INTERNAL ERROR %s",
                       "clustering method should have been checked before");
             }
-        }
-    }
+        } /* did not use mBed */
+    } /* had to calculate tree (did not read from file) */
+
 
 #if USE_WEIGHTS
     /* derive sequence weights from tree
@@ -998,6 +1021,11 @@ Align(mseq_t *prMSeq,
 
 #endif
 
+    if (TRUE == prOpts->bPileup){
+        PileUp(prMSeq, prOpts->rHhalignPara, prOpts->iClustersizes);
+        return 0;
+    }
+
 
     /* Read backgrounds HMMs and store in prHMMs
      *
@@ -1068,7 +1096,7 @@ Align(mseq_t *prMSeq,
 #if INDIRECT_HMM 
             AlnToHMM(&rHMMLocal, prMSeq)
 #else
-            AlnToHMM2(&rHMMLocal, prMSeq->seq, prMSeq->nseqs)
+            AlnToHMM2(&rHMMLocal, prOpts->rHhalignPara, prMSeq->seq, prMSeq->nseqs)
 #endif
             ) {
             Log(&rLog, LOG_ERROR, "Couldn't convert aligned input sequences to HMM. Will try to continue");
@@ -1093,7 +1121,7 @@ Align(mseq_t *prMSeq,
 #if INDIRECT_HMM 
             AlnToHMM(&rHMMLocal, prMSeqProfile)
 #else 
-            AlnToHMM2(&rHMMLocal, prMSeqProfile->seq, prMSeqProfile->nseqs)
+            AlnToHMM2(&rHMMLocal, prOpts->rHhalignPara, prMSeqProfile->seq, prMSeqProfile->nseqs)
 #endif
             ) {
             Log(&rLog, LOG_ERROR, "Couldn't convert profile1 to HMM. Will try to continue");
@@ -1112,10 +1140,15 @@ Align(mseq_t *prMSeq,
     /* Determine progressive alignment order
      */
     if (TRUE == prMSeq->aligned) {
-        Log(&rLog, LOG_INFO, "%s %s",
-             "Input sequences are aligned.",
-             "Will use Kimura distances of aligned sequences.");
-        prOpts->iPairDistType = PAIRDIST_SQUIDID_KIMURA;
+        if ( (SEQTYPE_PROTEIN == prMSeq->seqtype) && (TRUE == prOpts->bUseKimura) ){
+            Log(&rLog, LOG_INFO, "%s %s",
+                "Input sequences are aligned.",
+                "Will use Kimura distances of aligned sequences.");
+            prOpts->iPairDistType = PAIRDIST_SQUIDID_KIMURA;
+        }
+        else {
+            prOpts->iPairDistType = PAIRDIST_SQUIDID;
+        }
     }
 
 #if 0
@@ -1125,14 +1158,24 @@ Align(mseq_t *prMSeq,
     if (OK != AlignmentOrder(&piOrderLR, &pdSeqWeights, prMSeq,
                              prOpts->iPairDistType,
                              prOpts->pcDistmatInfile, prOpts->pcDistmatOutfile,
-                             prOpts->iClusteringType,
-                             prOpts->pcGuidetreeInfile, prOpts->pcGuidetreeOutfile,
-                             prOpts->bUseMbed)) {
+                             prOpts->iClusteringType, prOpts->iClustersizes, 
+                             prOpts->pcGuidetreeInfile, prOpts->pcGuidetreeOutfile, prOpts->pcClustfile, 
+                             prOpts->bUseMbed, prOpts->bPercID)) {
         Log(&rLog, LOG_ERROR, "AlignmentOrder() failed. Cannot continue");
         return -1;
     }
 #endif
 
+    /* if max-hmm-iter is set < 0 then do not perform alignment 
+     * there is a problem/feature(?) that the un-aligned sequences are output 
+     */
+    if (prOpts->iMaxHMMIterations < 0){
+        Log(&rLog, LOG_VERBOSE,
+            "iMaxHMMIterations < 0 (%d), will not perform alignment", prOpts->iMaxHMMIterations);
+        return 0;
+    }
+
+
     /* Progressive alignment of input sequences. Order defined by
      * branching of guide tree (piOrderLR). Use optional
      * background HMM information (prHMMs[0..prOpts->iHMMInputFiles-1])
@@ -1191,7 +1234,8 @@ Align(mseq_t *prMSeq,
             char zcIntermediate[1000] = {0};
             char *pcFormat = "fasta";
             sprintf(zcIntermediate, "clustalo-aln-iter~%d~", iIterationCounter);
-            if (WriteAlignment(prMSeq, zcIntermediate, MSAFILE_A2M)) {
+#define LINE_WRAP 60
+            if (WriteAlignment(prMSeq, zcIntermediate, MSAFILE_A2M, LINE_WRAP)) {
                 Log(&rLog, LOG_ERROR, "Could not save alignment to %s", zcIntermediate);
                 return -1;
             }
@@ -1219,9 +1263,11 @@ Align(mseq_t *prMSeq,
                 CKFREE(pdSeqWeights);
             Log(&rLog, LOG_INFO, "Computing new guide tree (iteration step %d)");
             if (AlignmentOrder(&piOrderLR, &pdSeqWeights, prMSeq,
-                               PAIRDIST_SQUIDID_KIMURA /* override */, NULL, prOpts->pcDistmatOutfile,
-                               prOpts->iClusteringType, NULL, prOpts->pcGuidetreeOutfile,
-                               prOpts->bUseMbedForIteration)) {
+                               ((SEQTYPE_PROTEIN == prMSeq->seqtype) && (TRUE == prOpts->bUseKimura)) ? PAIRDIST_SQUIDID_KIMURA : PAIRDIST_SQUIDID, 
+                               NULL, prOpts->pcDistmatOutfile,
+                               prOpts->iClusteringType, prOpts->iClustersizes,
+                               NULL, prOpts->pcGuidetreeOutfile, prOpts->pcClustfile, 
+                               prOpts->bUseMbedForIteration, prOpts->bPercID)) {
                 Log(&rLog, LOG_ERROR, "AlignmentOrder() failed. Cannot continue");
                 return -1;
             }
@@ -1244,7 +1290,7 @@ Align(mseq_t *prMSeq,
 #if INDIRECT_HMM 
                 AlnToHMM(&rHMMLocal, prMSeq)
 #else
-                AlnToHMM2(&rHMMLocal, prMSeq->seq, prMSeq->nseqs)
+                AlnToHMM2(&rHMMLocal, prOpts->rHhalignPara, prMSeq->seq, prMSeq->nseqs)
 #endif
                 ) {
                 Log(&rLog, LOG_ERROR, "Couldn't convert alignment to HMM. Will stop iterating now...");
@@ -1316,6 +1362,22 @@ Align(mseq_t *prMSeq,
         }
     }
 
+    if (NULL != prOpts->pcPosteriorfile){
+
+        hmm_light rHMMLocal = {0};
+
+        if (OK !=
+#if INDIRECT_HMM 
+            AlnToHMM(&rHMMLocal, prMSeq)
+#else
+            AlnToHMM2(&rHMMLocal, prOpts->rHhalignPara, prMSeq->seq, prMSeq->nseqs)
+#endif
+            ) {
+            Log(&rLog, LOG_ERROR, "Couldn't convert alignment to HMM. Will not do posterior probabilities...");
+        }
+        PosteriorProbabilities(prMSeq, rHMMLocal, prOpts->rHhalignPara, prOpts->pcPosteriorfile);
+        FreeHMMstruct(&rHMMLocal);
+    }
     
     if (NULL != piOrderLR) {
         CKFREE(piOrderLR);
@@ -1388,4 +1450,158 @@ AlignProfiles(mseq_t *prMSeqProfile1,
 }
 /* end of AlignProfiles() */
 
+/**
+ * @brief read pseudo-count 'fudge' parameters from file
+ *
+ * @param[out] rHhalignPara_p
+ * structure that holds several hhalign parameters
+ * @param[in] pcPseudoFile
+ * name of file with pseudo-count information.
+ * format must be collection of pairs of lines where one line specifies name of parameter 
+ * (gapb,gapd,gape,gapf,gapg,gaph,gapi,pca,pcb,pcc,gapbV,gapdV,gapeV,gapfV,gapgV,gaphV,gapiV,pcaV,pcbV,pccV)
+ * followed by second line with the (double) value of this parameter.
+ * 
+ * order of parameters is not fixed, not all parameters have to be set
+ */
+int ReadPseudoCountParams(hhalign_para *rHhalignPara_p, char *pcPseudoFile){
 
+
+    FILE *pfIn = NULL;
+    char zcLine[1000] = {0};
+    char zcFudge[1000] = {0};
+    double dVal = 0.00;
+    char *pcToken = NULL;
+    char *pcEnd = NULL;
+
+    if (NULL == (pfIn = fopen(pcPseudoFile, "r"))){
+        Log(&rLog, LOG_FATAL, "File %s with pseudo-count parameters does not exist", pcPseudoFile);
+    }
+    else {
+      while (NULL != fgets(zcLine, 1000, pfIn)){
+
+            if (NULL == (pcToken = strtok(zcLine, " \040\t\n"))){
+                Log(&rLog, LOG_FATAL, "no token specifying pseudo-count parameter in file %s\n"
+                       , pcPseudoFile
+                       );
+            }
+            strcpy(zcFudge, pcToken);
+
+	    if (NULL == fgets(zcLine, 1000, pfIn)){
+            Log(&rLog, LOG_FATAL, "no line with parameter in file %s associated with %s\n"
+                , pcPseudoFile, zcFudge
+		     );
+	    }
+	    else {
+	      if (NULL == (pcToken = strtok(zcLine, " \040\t\n"))){
+              Log(&rLog, LOG_FATAL, "no token in 2nd line in file %s associated with %s\n"
+                       , pcPseudoFile, zcFudge
+                       );
+	      }
+	      else {
+                dVal = (double)strtod(pcToken, &pcEnd);
+                if ('\0' != *pcEnd){
+                    Log(&rLog, LOG_FATAL, "token in file %s associated with %s not valid double (%s)\n"
+                        , pcPseudoFile, zcFudge, pcToken
+                        );
+                }
+	      }
+	    }
+#if 0
+	    printf("%s:%s:%d: read file %s: fudge = %s, val = %f\n"
+		   , __FUNCTION__, __FILE__, __LINE__
+		   , pcPseudoFile, zcFudge, dVal
+		   );
+#endif	    
+	    
+	    if      (0 == strcmp(zcFudge, "gapb")){
+            rHhalignPara_p->gapb = dVal;
+	    }
+	    else if (0 == strcmp(zcFudge, "gapd")){
+            rHhalignPara_p->gapd = dVal;
+	    }
+	    else if (0 == strcmp(zcFudge, "gape")){
+            rHhalignPara_p->gape = dVal;
+	    }
+	    else if (0 == strcmp(zcFudge, "gapf")){
+            rHhalignPara_p->gapf = dVal;
+	    }
+	    else if (0 == strcmp(zcFudge, "gapg")){
+            rHhalignPara_p->gapg = dVal;
+	    }
+	    else if (0 == strcmp(zcFudge, "gaph")){
+            rHhalignPara_p->gaph = dVal;
+	    }
+	    else if (0 == strcmp(zcFudge, "gapi")){
+            rHhalignPara_p->gapi = dVal;
+	    }
+	    else if (0 == strcmp(zcFudge, "pca")){
+            rHhalignPara_p->pca = dVal;
+	    }
+	    else if (0 == strcmp(zcFudge, "pcb")){
+            rHhalignPara_p->pcb = dVal;
+	    }
+	    else if (0 == strcmp(zcFudge, "pcc")){
+            rHhalignPara_p->pcc = dVal;
+	    }
+	    else if (0 == strcmp(zcFudge, "gapbV")){
+            rHhalignPara_p->gapbV = dVal;
+	    }
+	    else if (0 == strcmp(zcFudge, "gapdV")){
+            rHhalignPara_p->gapdV = dVal;
+	    }
+	    else if (0 == strcmp(zcFudge, "gapeV")){
+            rHhalignPara_p->gapeV = dVal;
+	    }
+	    else if (0 == strcmp(zcFudge, "gapfV")){
+            rHhalignPara_p->gapfV = dVal;
+	    }
+	    else if (0 == strcmp(zcFudge, "gapgV")){
+            rHhalignPara_p->gapgV = dVal;
+	    }
+	    else if (0 == strcmp(zcFudge, "gaphV")){
+            rHhalignPara_p->gaphV = dVal;
+	    }
+	    else if (0 == strcmp(zcFudge, "gapiV")){
+            rHhalignPara_p->gapiV = dVal;
+	    }
+	    else if (0 == strcmp(zcFudge, "pcaV")){
+            rHhalignPara_p->pcaV = dVal;
+	    }
+	    else if (0 == strcmp(zcFudge, "pcbV")){
+            rHhalignPara_p->pcbV = dVal;
+	    }
+	    else if (0 == strcmp(zcFudge, "pccV")){
+            rHhalignPara_p->pccV = dVal;
+	    }
+	    else {
+            Log(&rLog, LOG_FATAL, "%s not a valid pseudo-count parameter\n"
+                "must be one of [gapb,gapd,gape,gapf,gapg,gaph,gapi,pca,pcb,pcc,gapbV,gapdV,gapeV,gapfV,gapgV,gaphV,gapiV,pcaV,pcbV,pccV]\n"
+                , zcFudge);
+	    } /* switched between parameters */
+
+      } /* while !EOF */
+      fclose(pfIn); pfIn = NULL;
+      
+    } /* there was a parameter file */
+#if 0
+    printf("%s:%s:%d: gapb=%g,gapd=%g,gape=%g,gapf=%g,gapg=%g,gaph=%g,gapi=%g,pca=%g,pcb=%g,pcc=%g\n"
+	   , __FUNCTION__, __FILE__, __LINE__
+           , rHhalignPara_p->gapb, rHhalignPara_p->gapd, rHhalignPara_p->gape, rHhalignPara_p->gapf, rHhalignPara_p->gapg, rHhalignPara_p->gaph, rHhalignPara_p->gapi, rHhalignPara_p->pca, rHhalignPara_p->pcb, rHhalignPara_p->pcc
+           );
+#endif
+#if 0
+    printf("%s:%s:%d: gapbV=%g,gapdV=%g,gapeV=%g,gapfV=%g,gapgV=%g,gaphV=%g,gapiV=%g,pcaV=%g,pcbV=%g,pccV=%g\n"
+	   , __FUNCTION__, __FILE__, __LINE__
+           , rHhalignPara_p->gapbV, rHhalignPara_p->gapdV, rHhalignPara_p->gapeV, rHhalignPara_p->gapfV, rHhalignPara_p->gapgV, rHhalignPara_p->gaphV, rHhalignPara_p->gapiV, rHhalignPara_p->pcaV, rHhalignPara_p->pcbV, rHhalignPara_p->pccV
+           );
+#endif
+
+
+    return 0;
+
+} /* this is the end of ReadPseudoCountParams() */
+
+
+/*
+ * EOF clustal-omega.c
+ */
