@@ -15,7 +15,7 @@
  ********************************************************************/
 
 /*
- *  RCS $Id: hhalign_wrapper.c 284 2013-06-12 10:10:11Z fabian $
+ *  RCS $Id: hhalign_wrapper.c 290 2013-09-20 15:18:12Z fabian $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -53,6 +53,29 @@ void SetDefaultHhalignPara(hhalign_para *prHhalignPara)
     prHhalignPara->iMacRamMB = 2048;  /* 2048 default|give 2GB to MAC algorithm */
     prHhalignPara->bIsDna = false; /* protein mode unless we say otherwise */	
     prHhalignPara->bIsRna = false;	
+    prHhalignPara->pca = -UNITY;
+    prHhalignPara->pcb = -UNITY;
+    prHhalignPara->pcc = -UNITY;
+    prHhalignPara->pcw = -UNITY;
+    prHhalignPara->gapb = -UNITY;
+    prHhalignPara->gapd = -UNITY;
+    prHhalignPara->gape = -UNITY;
+    prHhalignPara->gapf = -UNITY;
+    prHhalignPara->gapg = -UNITY;
+    prHhalignPara->gaph = -UNITY;
+    prHhalignPara->gapi = -UNITY;
+    prHhalignPara->pcaV = -UNITY;
+    prHhalignPara->pcbV = -UNITY;
+    prHhalignPara->pccV = -UNITY;
+    prHhalignPara->pcwV = -UNITY;
+    prHhalignPara->gapbV = -UNITY;
+    prHhalignPara->gapdV = -UNITY;
+    prHhalignPara->gapeV = -UNITY;
+    prHhalignPara->gapfV = -UNITY;
+    prHhalignPara->gapgV = -UNITY;
+    prHhalignPara->gaphV = -UNITY;
+    prHhalignPara->gapiV = -UNITY;
+
 }
 /*** end: SetDefaultHhalignPara()  ***/
 
@@ -77,8 +100,8 @@ SanitiseUnknown(mseq_t *mseq)
 {
 
     int iS; /* iterator for sequence */
-    int iR; /* iterator for residue */
-    int iLen; /* length of sequence */
+    /*int iR;*/ /* iterator for residue */
+    /*int iLen;*/ /* length of sequence */
     char *pcRes = NULL;
 
 
@@ -458,6 +481,343 @@ PrepareAlignment(mseq_t *mseq, char **ppcProfile1, char **ppcProfile2,
 } /*** end: PrepareAlignment() ***/
 
 
+/**
+ * @brief PosteriorProbabilities() calculates posterior probabilities 
+ *  of aligning a single sequences on-to an alignment containing this sequence
+ *
+ * @param[in] prMSeq
+ * holds the aligned sequences [in] 
+ * @param[in] rHMMalignment
+ * HMM of the alignment in prMSeq
+ * @param[in] rHhalignPara
+ * various parameters read from commandline, needed by hhalign()
+ * @param[in] pcPosteriorfile
+ * name of file into which posterior probability information is written
+ *
+ * @return score of the alignment FIXME what is this?
+ *
+ * @note the PP-loop can be parallelised easily FIXME
+ */
+
+int
+PosteriorProbabilities(mseq_t *prMSeq, hmm_light rHMMalignment, hhalign_para rHhalignPara, char *pcPosteriorfile)
+{
+
+    double dScore = 0.0;
+    int i;
+    int iS; /* iterator for sequences */
+    int iNseq = prMSeq->nseqs; /* number of sequences */
+    int iLenHMM = rHMMalignment.L; /* length of alignment */
+    int iViterbiCount = 0; /* counts how often Viterbi is triggered */
+    char **ppcCopy = NULL;
+    char **ppcRepresent = NULL;
+    char zcAux[10000] = {0};
+    char zcError[10000] = {0};
+    hhalign_scores *prHHscores = NULL;
+    FILE *pfPP = NULL;
+
+    pfPP = fopen(pcPosteriorfile, "w");
+    fprintf(pfPP, "#1.i\t2.name\t\t3.L1\t4.L2\t5.sum\t\t6.sum/L1\t7.HH\n");
+
+
+    prHHscores = CKCALLOC(iNseq, sizeof(hhalign_scores));
+    for (iS = 0; iS < iNseq; iS++){
+        memset(&(prHHscores[iS]), 0, sizeof(hhalign_scores));
+    }
+
+    ppcCopy         = CKCALLOC(1, sizeof(char *));
+    ppcCopy[0]      = CKCALLOC(2*iLenHMM, sizeof(char));
+    ppcRepresent    = CKCALLOC(1, sizeof(char *));
+    ppcRepresent[0] = CKCALLOC(2*iLenHMM, sizeof(char));
+
+    for (i = 0; i < iLenHMM; i++){
+        ppcRepresent[0][i] = rHMMalignment.seq[rHMMalignment.ncons][i+1];
+    }
+
+    /* FIXME: this loop can be parallelised, FS r288 */
+    iViterbiCount = 0;
+    for (iS = 0; iS < iNseq; iS++){
+
+        /* single sequences may very well contain leading/trailing gaps,
+         * this will trigger warning in Transfer:hhalignment-C.h */
+        char *pcIter = NULL;
+        for (pcIter = prMSeq->orig_seq[iS]; ('-' == *pcIter) || ('.' == *pcIter); pcIter++);
+        strcpy(ppcCopy[0], pcIter/*prMSeq->orig_seq[iS]*/);
+        for (pcIter = &ppcCopy[0][strlen(ppcCopy[0])-1]; ('-' == *pcIter) || ('.' == *pcIter); pcIter--);
+        pcIter++;
+        *pcIter = '\0';
+        
+        zcError[0] = '\0';
+        hhalign(ppcCopy, 1, NULL, 
+                ppcRepresent, 0, NULL, 
+                &dScore, &rHMMalignment, NULL, NULL, NULL, NULL,
+                rHhalignPara, &prHHscores[iS], 0/* debug */, FALSE /* Log-Level*/, 
+                zcAux, zcError);
+        if (NULL != strstr(zcError, "Viterbi")){
+            iViterbiCount++;
+        }
+
+    } /* 0 <= iS < iNseq */
+    Log(&rLog, LOG_INFO, "Viterbi algorithm triggered %d times (out of %d)", iViterbiCount, iNseq-1);
+
+    for (iS = 0; iS < iNseq; iS++){
+
+        fprintf(pfPP, "%d\t%10s\t%3d"
+               , iS, prMSeq->sqinfo[iS].name, (int)(strlen(prMSeq->orig_seq[iS])));
+        fprintf(pfPP, "\t%3d\t%f\t%f\t%f", 
+                prHHscores[iS].L, prHHscores[iS].sumPP, prHHscores[iS].sumPP/strlen(prMSeq->orig_seq[iS]), prHHscores[iS].hhScore);
+        fprintf(pfPP, "\n");
+    } /* 0 <= iS < iNseq */
+
+
+
+    fclose(pfPP); pfPP = NULL;
+    free(ppcRepresent[0]); ppcRepresent[0] = NULL;
+    free(ppcCopy[0]); ppcCopy[0] = NULL;
+    free(ppcRepresent); ppcRepresent = NULL;
+    free(ppcCopy); ppcCopy = NULL;
+    free(prHHscores); prHHscores = NULL;
+
+    return 1;
+
+} /* this is the end of PosteriorProbabilities() */
+
+
+/**
+ * @brief sequentially align (chain) sequences
+ *
+ * @param[in,out] prMSeq
+ * holds the un-aligned sequences (in) and the final alignment (out)
+ * @param[in] rHhalignPara
+ * various parameters needed by hhalign()
+ * @param[in] iClustersize
+ * parameter that controls how often HMM is updated
+ *
+ * @note chained alignment takes much longer than balanced alignment
+ * because at every step ClustalO has to scan all previously aligned residues.
+ * for a balanced tree this takes N*log(N) time but for a chained tree 
+ * it takes N^2 time.
+ * This function has a short-cut, that the HMM need not be updated 
+ * for every single alignment step, but the HMM from the previous 
+ * step(s) can be re-cycled. The HMM is updated (i) at the very first step, 
+ * (ii) if a gap has been inserted into the HMM during alignment or 
+ * (iii) if the HMM has been used for too many steps without having 
+ * been updated. This update-frequency is controlled by the input 
+ * parameter iClustersize. iClustersize is the number of sequences used 
+ * to build a HMM to allow for one non-updating step. For example, 
+ * if iClustersize=100 and a HMM has been build from 100 sequences, 
+ * then this HMM can be used once without updating. If the HMM has 
+ * been built from 700 sequences (and iClustersize=100) then this 
+ * HMM can be used 7-times without having to be updated, etc. 
+ * For this reason the initial iClustersize sequences are always 
+ * aligned with fully updated HMMs. 
+ */
+double 
+PileUp(mseq_t *prMSeq, hhalign_para rHhalignPara, int iClustersize)
+{
+    /* @<variables local to PileUp> */
+    int iI; /* iterator for sequences */
+    int iCountPro = 0; /* count how many sequences are already in profile */
+    int *piLeafListSeq = NULL; /* lists required by PrepareAlignment() for attaching sequences pointers to ppcProfileSeq */
+    int *piLeafListPro = NULL; /* -"-  ppcProfilePro */
+    double *pdWeightL = NULL; /* argument used by hhalign, not used here */
+    double *pdWeightR = NULL; /* -"- */
+    double *pdWeightP = NULL; /* -"- */
+    char **ppcProfileSeq = NULL; /* pointer to sequences in profile */
+    char **ppcProfilePro = NULL; /* pointer to sequences in profile */
+    double dScore = -1.0; /* alignment score, calculated by hhalign() */
+    hhalign_scores rHHscores = {0}; /* more explicit scores than dScore */
+    hmm_light *prHMM = NULL; /* HMM against which to align single sequence */
+    hmm_light rHMMprofile = {0}; /* HMM calculated externally from profile */
+    int iHHret = -1; /* return value of hhalign() */
+    char zcAux[10000] = {0}; /* auxilliary print string used by hhalign() */
+    char zcError[10000] = {0}; /* error message printed by hhalign() */
+    char *pcConsensPro = NULL, /* auxilliary sequence strings required by hhalign() */
+        *pcReprsntPro = NULL, /* not used here */
+        *pcConsensSeq = NULL, *pcReprsntSeq = NULL;
+    bool bHMM_stale = YES; /* flag if HMM has to be updated */
+    int iSinceLastUpdate = -1; /* counts how often HMM has been used since last update */
+    progress_t *prProgress; /* structure required for progress report */
+    bool bPrintCR = (rLog.iLogLevelEnabled<=LOG_VERBOSE) ? FALSE : TRUE; /* progress report */
+    char **ppcReprsnt = NULL; /* string used to represent HMM */
+    ppcReprsnt    = CKCALLOC(1, sizeof(char *));
+
+    /* weights are not really used, but hhalign() expects them */
+    pdWeightL = (double *)CKMALLOC(prMSeq->nseqs * sizeof(double));
+    pdWeightR = (double *)CKMALLOC(prMSeq->nseqs * sizeof(double));
+    pdWeightP = (double *)CKMALLOC(prMSeq->nseqs * sizeof(double));
+
+    /* lists used for attaching ppcProfileSeq/ppcProfilePro to prMSeq */
+    piLeafListSeq = (int *)CKMALLOC(1 * sizeof(int));
+    piLeafListPro = (int *)CKMALLOC(prMSeq->nseqs * sizeof(int));
+
+    ppcProfileSeq    = (char **)CKMALLOC(1 * sizeof(char *));
+    ppcProfileSeq[0] = NULL;
+    ppcProfilePro    = (char **)CKMALLOC(prMSeq->nseqs * sizeof(char *));
+    for (iI = 0; iI < prMSeq->nseqs; iI++){
+        ppcProfilePro[iI] = NULL;
+    }
+    
+    piLeafListPro[0] = 0; /* first sequences in profile is simply un-aligned sequence */
+    iCountPro = 1; /* 'profile' now contains 1 sequence */
+
+    NewProgress(&prProgress, LogGetFP(&rLog, LOG_INFO),
+                "Progressive alignment progress", bPrintCR);
+
+    /* build up the initial alignment:
+     * sequences are chained but hhalign() is used normally, 
+     * that is, one sequence is aligned to a profile */
+    for (iI = 1; iI < MIN(prMSeq->nseqs, iClustersize); iI++){
+
+        piLeafListSeq[0] = iI;
+
+        /* PrepareAlignment() connects ppcProfileSeq/ppcProfilePro and prMSeq */
+        PrepareAlignment(prMSeq, ppcProfilePro, ppcProfileSeq,
+                         pdWeightL, pdWeightR, pdWeightP, 
+                         iI, piLeafListPro,
+                         1, piLeafListSeq);
+
+        /* align 1 (5th argument) sequence to the growing profile 
+         * (2nd argument number of sequences is iI), 
+         * external HMM not used at this stage (prHMM=NULL) */
+        iHHret = hhalign(ppcProfilePro, iI, NULL/*pdWeightL*/,
+                         ppcProfileSeq,  1, NULL/*pdWeightR*/,
+                         &dScore, prHMM,
+                         pcConsensPro, pcReprsntPro,
+                         pcConsensSeq, pcReprsntSeq,
+                         rHhalignPara, &rHHscores,
+                         CALL_FROM_REGULAR/* DEBUG ARGUMENT */, rLog.iLogLevelEnabled,
+                         zcAux, zcError);
+
+        piLeafListPro[iI] = iI;
+        iCountPro = iI+1;
+        ProgressLog(prProgress, iI, prMSeq->nseqs-1, FALSE); /* FIXME: test! FS, r288 */
+
+    } /* 1 <= iI < MIN(prMSeq->nseqs, iClustersize) */
+
+    /* now align single sequences not to a profile but to a HMM of a potentially old profile */
+    while (iI < prMSeq->nseqs){
+
+        /* if HMM not up-to-date then create new HMM for profile:
+         * HMM is stale (i) at the beginning, 
+         * (ii) if a gap has been inserted into the HMM during previous step,
+         * (iii) too many steps after the last up-date */
+        if ( (YES == bHMM_stale) ){
+            FreeHMMstruct(&rHMMprofile);
+            if (OK !=
+                AlnToHMM2(&rHMMprofile, rHhalignPara, prMSeq->seq, iI)
+                ) {
+                Log(&rLog, LOG_ERROR, "Couldn't convert alignment to HMM. Will not proceed with chained alignment, step %d", iI);
+            }
+            bHMM_stale = NO;
+            iSinceLastUpdate = 0;
+        }
+
+        /* align single sequence to HMM */
+        piLeafListSeq[0] = iI;
+
+        PrepareAlignment(prMSeq, ppcProfilePro, ppcProfileSeq,
+                         pdWeightL, pdWeightR, pdWeightP, 
+                         iI, piLeafListPro,
+                         1, piLeafListSeq);
+
+        /* use representative sequence to track where gaps are inserted into HMM */
+        ppcReprsnt[0] = CKREALLOC(ppcReprsnt[0], strlen(ppcProfilePro[0])+strlen(ppcProfileSeq[0])+1);
+        memcpy(ppcReprsnt[0], rHMMprofile.seq[rHMMprofile.ncons]+1, rHMMprofile.L);
+        ppcReprsnt[0][rHMMprofile.L] = '\0';
+
+        /* align 1 (5th argument) sequence to a HMM (2nd argument number of sequences is '0', 
+         * not iI as for a profile or '1' for one representative sequence), 
+         * external HMM (rHMMprofile calculated by AlnToHMM2()) is used at this stage */
+        prHMM = &rHMMprofile;
+        hhalign(ppcReprsnt, 0, NULL,
+                ppcProfileSeq, 1, NULL, 
+                &dScore, prHMM,
+                pcConsensPro, pcReprsntPro,
+                pcConsensSeq, pcReprsntSeq,
+                rHhalignPara, &rHHscores,
+                CALL_FROM_ALN_HMM/* DEBUG ARGUMENT */, rLog.iLogLevelEnabled,
+                zcAux, zcError);
+        iSinceLastUpdate++; /* HMM has been used one more time */
+        ProgressLog(prProgress, iI, prMSeq->nseqs-1, FALSE); /* FIXME: test! FS, r288 */
+
+        /* check if gaps have been inserted into HMM.
+         * if this is the case then HMM is stale 
+         * and the profile used to create the HMM must get gaps at the same positions.
+         * gaps are shown in the representative sequence */
+        if (rHMMprofile.L != strlen(ppcReprsnt[0])){
+
+            int iSeq; /* iterate through sequences of profile */
+
+            /* manually insert gaps into existing profile, 
+             * there should already be enough memory */
+#ifdef HAVE_OPENMP
+            /* all sequences in profile obtain gaps at the same position, 
+             * can do this in parallel */
+	        #pragma omp parallel for 
+#endif
+            for (iSeq = 0; iSeq < iI; iSeq++){
+
+                int iPtrRep, /* points to 'residue' in representative sequence */
+                    iPtrPrf; /* points to the corresponding position in the profile */
+
+                for (iPtrRep = strlen(ppcReprsnt[0])-1, iPtrPrf = strlen(ppcProfilePro[iSeq])-1; 
+                     iPtrRep >= 0; iPtrRep--){
+
+                    if ('-' == ppcReprsnt[0][iPtrRep]){ /* gaps are newly introduced into profile */
+                        ppcProfilePro[iSeq][iPtrRep] = '-';
+                    }
+                    else { /* non-gap characters (residues) are shifted towards the back */
+                        ppcProfilePro[iSeq][iPtrRep] = ppcProfilePro[iSeq][iPtrPrf];
+                        iPtrPrf--;
+                    }
+                    if (iPtrRep == iPtrPrf){ 
+                        /* if profile and representative seq point to same position then no more gaps */
+                        break; 
+                    }
+                } /* strlen(ppcReprsnt[0])-1 >= iPtrRep >= 0 */
+                ppcProfilePro[iSeq][strlen(ppcReprsnt[0])] = '\0';
+
+            } /* 0 <= iSeq < iI */
+            
+            /* make HMM stale */
+            bHMM_stale = YES;
+
+        } /* strlen(represent) != HMM.L (gaps inserted into HMM) */
+
+        /* check if maximum number of sequences exceeded */
+        if ( (iClustersize <= 1) || (iSinceLastUpdate > (double)(rHMMprofile.N_in)/(double)(iClustersize)) ){
+            bHMM_stale = YES;
+        }
+
+
+        piLeafListPro[iI] = iI;
+        iCountPro = iI+1;
+
+        iI++;
+
+    } /* iI < prMSeq->nseqs */
+    ProgressDone(prProgress);
+    FreeProgress(&prProgress);
+
+    FreeHMMstruct(&rHMMprofile);
+    CKFREE(pdWeightL); CKFREE(pdWeightR); CKFREE(pdWeightP); 
+    if (NULL != ppcReprsnt){ 
+        if (NULL != ppcReprsnt[0]){ 
+            CKFREE(ppcReprsnt[0]); 
+        }
+        CKFREE(ppcReprsnt); 
+    }
+    CKFREE(piLeafListSeq); CKFREE(piLeafListPro);
+
+    return 0.0;
+
+} /* this is the end of PileUp() */
+
+
+
+
+
 /**
  * @brief wrapper for hhalign. This is a frontend function to
  * the ported hhalign code.
@@ -502,6 +862,7 @@ PrepareAlignment(mseq_t *mseq, char **ppcProfile1, char **ppcProfile2,
  * @note: if hhalign() fails then try with Viterbi by setting MAC-RAM=0; FS, r241 -> r243
  */
 
+
 double
 HHalignWrapper(mseq_t *prMSeq, int *piOrderLR,
                double *pdSeqWeights, int iNodeCount,
@@ -598,6 +959,7 @@ HHalignWrapper(mseq_t *prMSeq, int *piOrderLR,
     NewProgress(&prProgress, LogGetFP(&rLog, LOG_INFO),
                 "Progressive alignment progress", bPrintCR);
 
+
     
     /* Profile-profile alignment? Then setup piLeafCount and
      * piLeafList here. FIXME this is just an awful haaaack
@@ -923,7 +1285,6 @@ HHalignWrapper(mseq_t *prMSeq, int *piOrderLR,
                     Log(&rLog, LOG_FATAL, "strlen(prof1)=%d, strlen(prof2)=%d -- nothing to align\n", 
                         iLen1, iLen2);
                 }
-                   
 
                 if (iLen1 < iLen2){
                     int iHHret = 0;
@@ -938,7 +1299,7 @@ HHalignWrapper(mseq_t *prMSeq, int *piOrderLR,
                                      zcAux, zcError);
 
                     if (RETURN_OK != iHHret){ /* FS, r241 -> */
-                        //fprintf(stderr, "%s:%d: emergency EXIT\n", __FILE__, __LINE__); exit(-1);
+                        /*fprintf(stderr, "%s:%d: emergency EXIT\n", __FILE__, __LINE__); exit(-1);*/
                         fprintf(stderr, 
                                 "%s:%s:%d: problem in alignment (profile sizes: %d + %d) (%s + %s), forcing Viterbi\n"
                                 "\thh-error-code=%d (mac-ram=%d)\n%s",
@@ -1000,7 +1361,7 @@ HHalignWrapper(mseq_t *prMSeq, int *piOrderLR,
                                      zcAux, zcError);
 
                     if (RETURN_OK != iHHret){ /* FS, r241 -> r243 */
-                        //fprintf(stderr, "%s:%d: emergency EXIT\n", __FILE__, __LINE__); exit(-1);
+                        /*fprintf(stderr, "%s:%d: emergency EXIT\n", __FILE__, __LINE__); exit(-1);*/
                         fprintf(stderr, 
                                 "%s:%s:%d: problem in alignment (profile sizes: %d + %d) (%s + %s), forcing Viterbi\n"
                                 "\thh-error-code=%d (mac-ram=%d)\n%s",
