@@ -15,7 +15,7 @@
  ********************************************************************/
 
 /*
- *  RCS $Id: pair_dist.c 283 2013-06-10 17:42:14Z fabian $
+ *  RCS $Id: pair_dist.c 288 2013-07-29 13:15:50Z andreas $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -290,10 +290,6 @@ PairDistances(symmatrix_t **distmat, mseq_t *mseq, int pairdist_type, bool bPerc
     assert(istart<iend);
     assert(jstart<jend);
 
-    if (NewSymMatrix(distmat, iend, jend)!=0) {
-        Log(&rLog, LOG_FATAL, "%s", "Memory allocation for distance matrix failed");
-    }
-
 
     /* compute pairwise distances or read from file
      *
@@ -302,10 +298,18 @@ PairDistances(symmatrix_t **distmat, mseq_t *mseq, int pairdist_type, bool bPerc
 #include "random-dist.h"
 #else
     if (NULL != fdist_in) {
-        Log(&rLog, LOG_FATAL, "FIXME: reading of distance matrix from file not implemented");
+        Log(&rLog, LOG_WARN,
+            "Please use distance matrix input only, if you know exactly what you're doing!");
+        if (SymMatrixRead(fdist_in, distmat, mseq)) {
+            Log(&rLog, LOG_FATAL, "%s", "Reading distance matrix failed");
+        }
 
     } else {
 
+        if (NewSymMatrix(distmat, iend, jend)!=0) {
+            Log(&rLog, LOG_FATAL, "%s", "Memory allocation for distance matrix failed");
+        }
+
         /* break into chunks, one for each thread
            matrix is a triangle, not a square
            hence making even chunk sizes is slightly fiddlier
