@@ -13,7 +13,7 @@
  ********************************************************************/
 
 /*
- *  RCS $Id: pair_dist.h 193 2011-02-07 15:45:21Z andreas $
+ *  RCS $Id: pair_dist.h 283 2013-06-10 17:42:14Z fabian $
  */
 
 
@@ -33,7 +33,7 @@
 #include "symmatrix.h"
 
 extern int
-PairDistances(symmatrix_t **distmat, mseq_t *mseq, const int pairdist_type,
+PairDistances(symmatrix_t **distmat, mseq_t *mseq, const int pairdist_type, bool bPercID, 
               const int istart, const int iend,
               const int jstart, const int jend,
               char *fdist_in, char *fdist_out);
