@@ -15,7 +15,7 @@
  ********************************************************************/
 
 /*
- *  RCS $Id: seq.c 283 2013-06-10 17:42:14Z fabian $
+ *  RCS $Id: seq.c 291 2014-02-27 18:20:54Z fabian $
  *
  *
  * Module for sequence/alignment IO and misc.
@@ -65,7 +65,7 @@ AliStat(mseq_t *prMSeq, bool bSampling, bool bReportAll) {
      */
     float  **ppdIdentMx;  /* identity matrix (squid: imx) */
     const int iNumSample = 1000; /* sample size (squid: nsample) */
-    int iAux;
+
 
     MSA *msa; /* squid's alignment structure */
     int small, large;	
@@ -418,7 +418,7 @@ SeqTypeToStr(int iSeqType)
  */
 int
 ReadSequences(mseq_t *prMSeq, char *seqfile, 
-              int iSeqType, int iSeqFmt, bool bIsProfile, 
+              int iSeqType, int iSeqFmt, bool bIsProfile, bool bDealignInputSeqs, 
               int iMaxNumSeq, int iMaxSeqLen)
 {
     SQFILE *dbfp; /* sequence file descriptor */
@@ -561,7 +561,7 @@ ReadSequences(mseq_t *prMSeq, char *seqfile,
 #endif*/
     
     /* Check if sequences are aligned */
-    prMSeq->aligned = SeqsAreAligned(prMSeq, bIsProfile);
+    prMSeq->aligned = SeqsAreAligned(prMSeq, bIsProfile, bDealignInputSeqs);
 
     
     /* keep original sequence as copy and convert "working" sequence
@@ -886,15 +886,15 @@ WriteAlignment(mseq_t *mseq, const char *pcAlnOutfile, int outfmt, int iWrap, bo
     
     switch (outfmt) {
     case MSAFILE_A2M:
-        //WriteA2M(pfOut, msa, 0);
+        /*WriteA2M(pfOut, msa, 0);*/
         WriteA2M(pfOut, msa, iWrap);
         break;
     case MSAFILE_VIENNA:
-        //WriteA2M(pfOut, msa, 1);
+        /*WriteA2M(pfOut, msa, 1);*/
         WriteA2M(pfOut, msa, INT_MAX);
         break;
     case MSAFILE_CLUSTAL:
-        WriteClustal(pfOut, msa, iWrap, TRUE==bResno ? 1 : 0);
+        WriteClustal(pfOut, msa, iWrap, TRUE==bResno ? 1 : 0, mseq->seqtype);
         break;
     case MSAFILE_MSF:
         WriteMSF(pfOut, msa);
@@ -1021,7 +1021,7 @@ SortMSeqByLength(mseq_t *prMSeq, const char cOrder)
  * 
  */    
 bool
-SeqsAreAligned(mseq_t *prMSeq, bool bIsProfile)
+SeqsAreAligned(mseq_t *prMSeq, bool bIsProfile, bool bDealignInputSeqs)
 {
     bool bGapFound, bSameLength;
     int iSeqIdx; /* sequence counter */
@@ -1069,11 +1069,40 @@ SeqsAreAligned(mseq_t *prMSeq, bool bIsProfile)
 #if 0
     Log(&rLog, LOG_FORCED_DEBUG, "bSameLength=%d bGapFound=%d", bSameLength, bGapFound);
 #endif
+
+#if 0
     if ( (TRUE == bSameLength) && ((TRUE == bGapFound) || (TRUE == bIsProfile)) ) {
         return TRUE;
     } else {
+        if ((FALSE == bSameLength) && (TRUE == bGapFound) && (FALSE == bDealignInputSeqs)){
+            Log(&rLog, LOG_FORCED_DEBUG, "Potential Problem: Gaps encountered but not all sequences have same length, consider using --dealign");
+        }
         return FALSE;
     }   
+#else
+    if (FALSE == bSameLength){
+        /* if sequences don't have same lengths they can never be profile */
+        if (TRUE == bGapFound){
+            Log(&rLog, LOG_FORCED_DEBUG, "Potential Problem: sequences (N=%d) don't have same lengths but contain gaps, consider using --dealign", prMSeq->nseqs);
+        }
+        return FALSE;
+    }
+    else { /* here all sequences have same lengths */
+        if (TRUE == bGapFound){
+            /* if at least one sequence contains gaps (and all have the same lengths) 
+               then we can be sure it is a profile */
+            return TRUE;
+        }
+        /* here all sequences have same lengths but no sequences contain any gaps */
+        else if (TRUE == bIsProfile){
+            /* if the user says it is a profile then it is */
+            return TRUE;
+        }
+        else {
+            return FALSE;
+        }
+    }
+#endif
 
 } 
 /***   end: SeqsAreAligned()   ***/
@@ -1205,10 +1234,16 @@ JoinMSeqs(mseq_t **prMSeqDest_p, mseq_t *prMSeqToAdd)
 
     (*prMSeqDest_p)->nseqs = iNewNSeq;
 
+#if 0
     /* 2nd arg is bIsProfile, which when set TRUE skips 
      * the check for gaps. here always check for gaps, 
      * so set FALSE (main reason is that it is easier), FS, r282 -> */    
-    (*prMSeqDest_p)->aligned = SeqsAreAligned(*prMSeqDest_p, FALSE);
+    /* had a problem at this stage, therefore dispense with gap check, FS, r290 -> */
+    /* 3rd argument is dealignment flag, do not dealign profiles */
+    (*prMSeqDest_p)->aligned = SeqsAreAligned(*prMSeqDest_p, TRUE/*FALSE*/, FALSE);
+#else 
+	(*prMSeqDest_p)->aligned = TRUE;
+#endif
     
     return; 
 }
