@@ -15,7 +15,7 @@
  ********************************************************************/
 
 /*
- *  RCS $Id: hhalign.cpp 284 2013-06-12 10:10:11Z fabian $
+ *  RCS $Id: hhalign.cpp 290 2013-09-20 15:18:12Z fabian $
  */
 
 /* hhalign.C: 
@@ -714,7 +714,7 @@ hhalign(char **ppcFirstProf, int iFirstCnt, double *pdWeightsL,
     int Nali;                    /* number of normally backtraced alignments 
                                     in dot plot */
     
-    SetDefaults(); 
+    SetDefaults(rHhalignPara); 
     strcpy(par.tfile,""); /* FS, Initialise Argument */
     strcpy(par.alnfile,""); 
     par.p=0.0 ;                  /* minimum threshold for inclusion in hit list 
@@ -848,13 +848,13 @@ hhalign(char **ppcFirstProf, int iFirstCnt, double *pdWeightsL,
 	if(rHhalignPara.bIsDna)
 	{
 		nucleomode = true;
-		SetDnaDefaults();
+		SetDnaDefaults(rHhalignPara);
 		SetDnaSubstitutionMatrix();
 	}
 	else if(rHhalignPara.bIsRna)
 	{
 		nucleomode = true;
-		SetRnaDefaults();
+		SetRnaDefaults(rHhalignPara);
 		SetRnaSubstitutionMatrix();
 	}	
 	else
@@ -889,6 +889,8 @@ hhalign(char **ppcFirstProf, int iFirstCnt, double *pdWeightsL,
     // longest allowable length of database HMM
     int Lmaxmem=(int)((float)MEMSPACE_DYNPROG/qL/6/8); 
     if (par.forward==2 && tL+2>=Lmaxmem) {
+        sprintf(zcError, "%s:%s:%d: switch to Viterbi (qL=%d, tL=%d, MAC-RAM=%d\n)", 
+                __FUNCTION__, __FILE__, __LINE__, qL, tL, rHhalignPara.iMacRamMB);
         if (v>=1)
             cerr<<"WARNING: Not sufficient memory to realign with MAC algorithm. Using Viterbi algorithm."<<endl;
         par.forward=0;
@@ -1084,7 +1086,7 @@ hhalign(char **ppcFirstProf, int iFirstCnt, double *pdWeightsL,
                   hit.BacktraceMAC(q,t);
                   if ((isnan(hit.score)) || (isnan(hit.Pforward))){
                       printf("nan after backtrace\n");
-                  }
+                  } 
               } /* use MAC algorithm */
           //       printf ("%-12.12s  %-12.12s   irep=%-2i  score=%6.2f hit.Pvalt=%.2g\n",hit.name,hit.fam,hit.irep,hit.score,hit.Pvalt);
           *dScore_p = hit.score;
