@@ -15,7 +15,7 @@
  ********************************************************************/
 
 /*
- * RCS $Id: hhalign.h 267 2011-10-24 09:49:27Z fabian $
+ * RCS $Id: hhalign.h 284 2013-06-12 10:10:11Z fabian $
  */
 
 
@@ -26,5 +26,5 @@ hhalign(char **ppcFirstProf, int iFirstCnt, double *pdWeightsL,
         char *pcPrealigned1, char *pcRepresent1,
         char *pcPrealigned2, char *pcRepresent2,
         hhalign_para rHhalignPara, hhalign_scores *rHHscores, 
-        int iFlag,
+        int iFlag, int iVerbosity,
         char zcAux[], char zcError[]);
