@@ -15,7 +15,7 @@
  ********************************************************************/
 
 /*
- *  RCS $Id: hhfullalignment-C.h 243 2011-05-31 13:49:19Z fabian $
+ *  RCS $Id: hhfullalignment-C.h 284 2013-06-12 10:10:11Z fabian $
  */
 
 // hhfullalignment.C
@@ -166,7 +166,7 @@ FullAlignment::AddGaps()
  * @brief Build full alignment -> qa->s[k][h] and ta->s[k][h]
  */
 int
-FullAlignment::Build(HMM& q, Hit& hit)
+FullAlignment::Build(HMM& q, Hit& hit, char zcError[])
 {
   int step;
   char prev_state=MM, state=MM;  
@@ -225,7 +225,7 @@ FullAlignment::Build(HMM& q, Hit& hit)
     }
   }
   else {
-      fprintf(stderr, 
+      sprintf(zcError, 
               "+-------------------------------+\n"
               "| both sequences truncated left |\n"
               "+-------------------------------+\n"
@@ -290,7 +290,7 @@ FullAlignment::Build(HMM& q, Hit& hit)
     }
   }
   else{
-      fprintf(stderr, 
+      sprintf(zcError, 
               "+--------------------------------+\n"
               "| both sequences truncated right |\n"
               "+--------------------------------+\n"
