@@ -15,7 +15,7 @@
  ********************************************************************/
 
 /*
- * RCS $Id: hhhit-C.h 260 2011-09-01 16:10:46Z fabian $
+ * RCS $Id: hhhit-C.h 284 2013-06-12 10:10:11Z fabian $
  */
 
 // hhhit.C
@@ -378,11 +378,11 @@ Hit::Viterbi(HMM& q, HMM& t, float** Sstruc)
     float *sDG = new(float[par.maxResLen]);   // sDG[i][j] = score of best alignment up to indices (i,j) ending in (Delete,Gap)
     float *sIM = new(float[par.maxResLen]);   // sIM[i][j] = score of best alignment up to indices (i,j) ending in (Ins,Match)
     float *sMI = new(float[par.maxResLen]);   // sMI[i][j] = score of best alignment up to indices (i,j) ending in (Match,Ins) 
-  float smin=(par.loc? 0:-FLT_MAX);  //used to distinguish between SW and NW algorithms in maximization         
-  int i,j;      //query and template match state indices
-  float sMM_i_j=0,sMI_i_j,sIM_i_j,sGD_i_j,sDG_i_j;
-  float sMM_i_1_j_1,sMI_i_1_j_1,sIM_i_1_j_1,sGD_i_1_j_1,sDG_i_1_j_1;
-  int jmin,jmax;
+    float smin=(par.loc? 0:-FLT_MAX);  //used to distinguish between SW and NW algorithms in maximization         
+    int i=0,j=0;      //query and template match state indices
+    float sMM_i_j=0, sMI_i_j=0, sIM_i_j=0, sGD_i_j=0, sDG_i_j=0;
+    float sMM_i_1_j_1=0, sMI_i_1_j_1=0, sIM_i_1_j_1=0, sGD_i_1_j_1=0, sDG_i_1_j_1=0;
+    int jmin=0, jmax=0;
 
   // Reset crossed out cells?
   if(irep==1) InitializeForAlignment(q,t);
@@ -1677,59 +1677,62 @@ Hit::InitializeForAlignment(HMM& q, HMM& t)
 void 
 Hit::InitializeBacktrace(HMM& q, HMM& t)
 {
-  if (irep==1) //if this is the first single repeat repeat hit with this template
-    {
-      //Copy information about template profile to hit and reset template pointers to avoid destruction
-      longname=new(char[strlen(t.longname)+1]);
-      name    =new(char[strlen(t.name)+1]);
-      file    =new(char[strlen(t.file)+1]);
-      if (!file) MemoryError("space for alignments with database HMMs. \nNote that all alignments have to be kept in memory");
-      strcpy(longname,t.longname);
-      strcpy(name,t.name);
-      strcpy(fam ,t.fam);
-      strcpy(sfam ,t.sfam);
-      strcpy(fold ,t.fold);
-      strcpy(cl ,t.cl);
-      strcpy(file,t.file);
-      sname=new(char*[t.n_display]);   // Call Compare only once with irep=1
-      seq  =new(char*[t.n_display]);   // Call Compare only once with irep=1
-      if (!sname || !seq) 
-	MemoryError("space for alignments with database HMMs.\nNote that all sequences for display have to be kept in memory");
+    if (irep==1) //if this is the first single repeat repeat hit with this template
+        {
+            //Copy information about template profile to hit and reset template pointers to avoid destruction
+            longname=new(char[strlen(t.longname)+1])();
+            name    =new(char[strlen(t.name)+1])();
+            file    =new(char[strlen(t.file)+1])();
+            if (!file) {
+                MemoryError("space for alignments with database HMMs. \nNote that all alignments have to be kept in memory");
+            }
+            strcpy(longname,t.longname);
+            strcpy(name,t.name);
+            strcpy(fam ,t.fam);
+            strcpy(sfam ,t.sfam);
+            strcpy(fold ,t.fold);
+            strcpy(cl ,t.cl);
+            strcpy(file,t.file);
+            sname=new(char*[t.n_display])();   // Call Compare only once with irep=1
+            seq  =new(char*[t.n_display])();   // Call Compare only once with irep=1
+            if (!sname || !seq) {
+                MemoryError("space for alignments with database HMMs.\nNote that all sequences for display have to be kept in memory");
+            }
 
-      for (int k=0; k<t.n_display; k++)	{
-          if (NULL != t.sname){
-              sname[k]=t.sname[k]; t.sname[k]=NULL;
-          }
-          else {
-              sname[k]=NULL;
-          }
-          seq[k]  =t.seq[k];   t.seq[k]=NULL;
-      }
-
-      n_display=t.n_display; t.n_display=0;
-      ncons  = t.ncons;
-      nfirst = t.nfirst;
-      nss_dssp = t.nss_dssp;
-      nsa_dssp = t.nsa_dssp;
-      nss_pred = t.nss_pred;
-      nss_conf = t.nss_conf;
-      L = t.L;
-      Neff_HMM = t.Neff_HMM;
-      Eval   = 1.0;
-      Pval   = 1.0;
-      Pvalt  = 1.0;
-      logPval = 0.0;
-      logPvalt= 0.0;
-      Probab = 1.0;
-    }    
-
-  // Allocate new space
-  this->i = new( int[i2+j2+2]);
-  this->j = new( int[i2+j2+2]);
-  states  = new( char[i2+j2+2]);
-  S = S_ss = P_posterior = NULL; // set to NULL to avoid deleting data from irep=1 when hit with irep=2 is removed 
-  Xcons = NULL;
-}
+            for (int k=0; k<t.n_display; k++)	{
+                if (NULL != t.sname){
+                    sname[k]=t.sname[k]; t.sname[k]=NULL;
+                }
+                else {
+                    sname[k]=NULL;
+                }
+                seq[k]  =t.seq[k];   t.seq[k]=NULL;
+            }
+            
+            n_display=t.n_display; t.n_display=0;
+            ncons  = t.ncons;
+            nfirst = t.nfirst;
+            nss_dssp = t.nss_dssp;
+            nsa_dssp = t.nsa_dssp;
+            nss_pred = t.nss_pred;
+            nss_conf = t.nss_conf;
+            L = t.L;
+            Neff_HMM = t.Neff_HMM;
+            Eval   = 1.0;
+            Pval   = 1.0;
+            Pvalt  = 1.0;
+            logPval = 0.0;
+            logPvalt= 0.0;
+            Probab = 1.0;
+        }    
+    
+    // Allocate new space
+    this->i = new( int[i2+j2+2])();
+    this->j = new( int[i2+j2+2])();
+    states  = new( char[i2+j2+2])();
+    S = S_ss = P_posterior = NULL; // set to NULL to avoid deleting data from irep=1 when hit with irep=2 is removed 
+    Xcons = NULL;
+} /* this is the end of Hit::InitializeBacktrace() */
 
 /////////////////////////////////////////////////////////////////////////////////////
 // Some score functions 
