@@ -15,7 +15,7 @@
  ********************************************************************/
 
 /*
- * RCS $Id: hhhmm-C.h 267 2011-10-24 09:49:27Z fabian $
+ * RCS $Id: hhhmm-C.h 284 2013-06-12 10:10:11Z fabian $
  */
 
 
@@ -60,40 +60,40 @@ using std::ofstream;
 //////////////////////////////////////////////////////////////////////////////
 HMM::HMM(int maxseqdis, int maxres)
 {
-  sname = new char*[maxseqdis];   // names of stored sequences 
-  for (int i = 0; i < maxseqdis; i++){ sname[i] = NULL;}
-  seq = new char*[maxseqdis];     // residues of stored sequences (first at pos 1!)
-  for (int i = 0; i < maxseqdis; i++){ seq[i] = NULL;}
-  Neff_M = new float[maxres];     // Neff_M[i] = diversity of subalignment of seqs that have residue in col i
-  Neff_I = new float[maxres];     // Neff_I[i] = diversity of subalignment of seqs that have insert in col i
-  Neff_D = new float[maxres];     // Neff_D[i] = diversity of subalignment of seqs that have delete in col i
-  longname = new char[DESCLEN];   // Full name of first sequence of original alignment (NAME field)
-  ss_dssp = new char[maxres];     // secondary structure determined by dssp 0:-  1:H  2:E  3:C  4:S  5:T  6:G  7:B
-  sa_dssp = new char[maxres];     // solvent accessibility state determined by dssp 0:-  1:A (absolutely buried) 2:B  3:C  4:D  5:E (exposed)
-  ss_pred = new char[maxres];     // predicted secondary structure          0:-  1:H  2:E  3:C
-  ss_conf = new char[maxres];     // confidence value of prediction         0:-  1:0 ... 10:9
-  Xcons   = NULL;                 // create only when needed: consensus sequence in internal representation (A=0 R=1 N=2 D=3 ...)
-  l = new int[maxres];            // l[i] = pos. of j'th match state in aligment
-  /* FS introduced sentinel, NULL terminates loop in destructor, FS, r221->222 */
-  f = new float*[maxres+1];   f[maxres] = NULL;  // f[i][a] = prob of finding amino acid a in column i WITHOUT pseudocounts
-  g = new float*[maxres+1];   g[maxres] = NULL;  // f[i][a] = prob of finding amino acid a in column i WITH pseudocounts
-  p = new float*[maxres+1];   p[maxres] = NULL;  // p[i][a] = prob of finding amino acid a in column i WITH OPTIMUM pseudocounts
-  tr = new float*[maxres+1]; tr[maxres] = NULL;  // log2 of transition probabilities M2M M2I M2D I2M I2I D2M D2D M2M_GAPOPEN GAPOPEN GAPEXTD
-//   tr_lin = new float*[maxres];    // linear transition probabilities M2M M2I M2D I2M I2I D2M D2D M2M_GAPOPEN GAPOPEN GAPEXTD
-  for (int i=0; i<maxres; i++) {f[i]=new(float[NAA+3]);}
-  for (int i=0; i<maxres; i++) {g[i]=new(float[NAA]);}  
-  for (int i=0; i<maxres; i++) {p[i]=new(float[NAA]);}  
-  for (int i=0; i<maxres; i++) {tr[i]=new(float[NTRANS]);}
-//   for (int i=0; i<maxres; i++) tr_lin[i]=new(float[NTRANS]);
-  L=0; 
-  Neff_HMM=0; 
-  n_display=N_in=N_filtered=0; 
-  nss_dssp=nsa_dssp=nss_pred=nss_conf=nfirst=ncons=-1;
-//   lamda_hash.New(37,0.0); // Set size and NULL element for hash
-//   mu_hash.New(37,0.0);    // Set size and NULL element for hash
-  lamda=0.0; mu=0.0;
-  name[0]=longname[0]=fam[0]='\0';
-  trans_lin=0; // transition probs in log space
+    sname = new char*[maxseqdis]();   // names of stored sequences 
+    for (int i = 0; i < maxseqdis; i++){ sname[i] = NULL;}
+    seq = new char*[maxseqdis]();     // residues of stored sequences (first at pos 1!)
+    for (int i = 0; i < maxseqdis; i++){ seq[i] = NULL;}
+    Neff_M = new float[maxres]();     // Neff_M[i] = diversity of subalignment of seqs that have residue in col i
+    Neff_I = new float[maxres]();     // Neff_I[i] = diversity of subalignment of seqs that have insert in col i
+    Neff_D = new float[maxres]();     // Neff_D[i] = diversity of subalignment of seqs that have delete in col i
+    longname = new char[DESCLEN]();   // Full name of first sequence of original alignment (NAME field)
+    ss_dssp = new char[maxres]();     // secondary structure determined by dssp 0:-  1:H  2:E  3:C  4:S  5:T  6:G  7:B
+    sa_dssp = new char[maxres]();     // solvent accessibility state determined by dssp 0:-  1:A (absolutely buried) 2:B  3:C  4:D  5:E (exposed)
+    ss_pred = new char[maxres]();     // predicted secondary structure          0:-  1:H  2:E  3:C
+    ss_conf = new char[maxres]();     // confidence value of prediction         0:-  1:0 ... 10:9
+    Xcons   = NULL;                 // create only when needed: consensus sequence in internal representation (A=0 R=1 N=2 D=3 ...)
+    l = new int[maxres]();            // l[i] = pos. of j'th match state in aligment
+    /* FS introduced sentinel, NULL terminates loop in destructor, FS, r221->222 */
+    f = new float*[maxres+1]();   f[maxres] = NULL;  // f[i][a] = prob of finding amino acid a in column i WITHOUT pseudocounts
+    g = new float*[maxres+1]();   g[maxres] = NULL;  // f[i][a] = prob of finding amino acid a in column i WITH pseudocounts
+    p = new float*[maxres+1]();   p[maxres] = NULL;  // p[i][a] = prob of finding amino acid a in column i WITH OPTIMUM pseudocounts
+    tr = new float*[maxres+1](); tr[maxres] = NULL;  // log2 of transition probabilities M2M M2I M2D I2M I2I D2M D2D M2M_GAPOPEN GAPOPEN GAPEXTD
+    //   tr_lin = new float*[maxres];    // linear transition probabilities M2M M2I M2D I2M I2I D2M D2D M2M_GAPOPEN GAPOPEN GAPEXTD
+    for (int i=0; i<maxres; i++) {f[i]=new(float[NAA+3])();}
+    for (int i=0; i<maxres; i++) {g[i]=new(float[NAA])();}  
+    for (int i=0; i<maxres; i++) {p[i]=new(float[NAA])();}  
+    for (int i=0; i<maxres; i++) {tr[i]=new(float[NTRANS])();}
+    //   for (int i=0; i<maxres; i++) tr_lin[i]=new(float[NTRANS]);
+    L=0; 
+    Neff_HMM=0; 
+    n_display=N_in=N_filtered=0; 
+    nss_dssp=nsa_dssp=nss_pred=nss_conf=nfirst=ncons=-1;
+    //   lamda_hash.New(37,0.0); // Set size and NULL element for hash
+    //   mu_hash.New(37,0.0);    // Set size and NULL element for hash
+    lamda=0.0; mu=0.0;
+    name[0]=longname[0]=fam[0]='\0';
+    trans_lin=0; // transition probs in log space
 }
 
 
