@@ -15,7 +15,7 @@
  ********************************************************************/
 
 /*
- * RCS $Id: hhutil-C.h 279 2013-05-16 15:55:15Z fabian $
+ * RCS $Id: hhutil-C.h 290 2013-09-20 15:18:12Z fabian $
  */
 
 #define PARAMETERSFROMFILE 0
@@ -547,7 +547,7 @@ ReadDefaultsFile(int& argc_conf, char** argv_conf)
  * @brief Set default parameter values
  */
 void 
-SetDefaults()
+SetDefaults(hhalign_para rHhalignPara)
 {
     
   par.append=0;                // overwrite output file
@@ -705,14 +705,36 @@ SetDefaults()
   par.exclstr=NULL; 
 
 
-#if PARAMETERSFROMFILE  /* read parameter file from home-dir */
-#include "hhutil-C-help.h"
-#endif /* read parameter file from home-dir */
+  //#if PARAMETERSFROMFILE  /* read parameter file from home-dir */
+  //#include "hhutil-C-help.h"
+  //#endif /* read parameter file from home-dir */
+  if (rHhalignPara.pca  >= 0.00){ par.pca  = rHhalignPara.pca;  }
+  if (rHhalignPara.pcb  >= 0.00){ par.pcb  = rHhalignPara.pcb;  }
+  if (rHhalignPara.pcc  >= 0.00){ par.pcc  = rHhalignPara.pcc;  }
+  if (rHhalignPara.pcw  >= 0.00){ par.pcw  = rHhalignPara.pcw;  }
+  if (rHhalignPara.gapb >= 0.00){ par.gapb = rHhalignPara.gapb; }
+  if (rHhalignPara.gapd >= 0.00){ par.gapd = rHhalignPara.gapd; }
+  if (rHhalignPara.gape >= 0.00){ par.gape = rHhalignPara.gape; }
+  if (rHhalignPara.gapf >= 0.00){ par.gapf = rHhalignPara.gapf; }
+  if (rHhalignPara.gapg >= 0.00){ par.gapg = rHhalignPara.gapg; }
+  if (rHhalignPara.gaph >= 0.00){ par.gaph = rHhalignPara.gaph; }
+  if (rHhalignPara.gapi >= 0.00){ par.gapi = rHhalignPara.gapi; }
+  if (rHhalignPara.pcaV  >= 0.00){ par.pcaV  = rHhalignPara.pcaV;  }
+  if (rHhalignPara.pcbV  >= 0.00){ par.pcbV  = rHhalignPara.pcbV;  }
+  if (rHhalignPara.pccV  >= 0.00){ par.pccV  = rHhalignPara.pccV;  }
+  if (rHhalignPara.pcwV  >= 0.00){ par.pcwV  = rHhalignPara.pcwV;  }
+  if (rHhalignPara.gapbV >= 0.00){ par.gapbV = rHhalignPara.gapbV; }
+  if (rHhalignPara.gapdV >= 0.00){ par.gapdV = rHhalignPara.gapdV; }
+  if (rHhalignPara.gapeV >= 0.00){ par.gapeV = rHhalignPara.gapeV; }
+  if (rHhalignPara.gapfV >= 0.00){ par.gapfV = rHhalignPara.gapfV; }
+  if (rHhalignPara.gapgV >= 0.00){ par.gapgV = rHhalignPara.gapgV; }
+  if (rHhalignPara.gaphV >= 0.00){ par.gaphV = rHhalignPara.gaphV; }
+  if (rHhalignPara.gapiV >= 0.00){ par.gapiV = rHhalignPara.gapiV; }
 
    return;
 } /** this is the end of SetDefaults() **/
 
-void SetRnaDefaults()
+void SetRnaDefaults(hhalign_para rHhalignPara)
 {
   par.pca=1.28f;                // default values for substitution matrix pseudocounts 
   par.pcb=1.75f;                // significant reduction of pcs by Neff_M starts around Neff_M-1=pcb
@@ -755,13 +777,35 @@ void SetRnaDefaults()
   par.gapiV=0.711993;                // factor for increasing gap extension penalty for inserts  
 #endif
 
-#if PARAMETERSFROMFILE  /* read parameter file from home-dir */
-#include "hhutil-C-help.h"
-#endif /* read parameter file from home-dir */
+  //#if PARAMETERSFROMFILE  /* read parameter file from home-dir */
+  //#include "hhutil-C-help.h"
+  //#endif /* read parameter file from home-dir */
+  if (rHhalignPara.pca  >= 0.00){ par.pca  = rHhalignPara.pca;  }
+  if (rHhalignPara.pcb  >= 0.00){ par.pcb  = rHhalignPara.pcb;  }
+  if (rHhalignPara.pcc  >= 0.00){ par.pcc  = rHhalignPara.pcc;  }
+  if (rHhalignPara.pcw  >= 0.00){ par.pcw  = rHhalignPara.pcw;  }
+  if (rHhalignPara.gapb >= 0.00){ par.gapb = rHhalignPara.gapb; }
+  if (rHhalignPara.gapd >= 0.00){ par.gapd = rHhalignPara.gapd; }
+  if (rHhalignPara.gape >= 0.00){ par.gape = rHhalignPara.gape; }
+  if (rHhalignPara.gapf >= 0.00){ par.gapf = rHhalignPara.gapf; }
+  if (rHhalignPara.gapg >= 0.00){ par.gapg = rHhalignPara.gapg; }
+  if (rHhalignPara.gaph >= 0.00){ par.gaph = rHhalignPara.gaph; }
+  if (rHhalignPara.gapi >= 0.00){ par.gapi = rHhalignPara.gapi; }
+  if (rHhalignPara.pcaV  >= 0.00){ par.pcaV  = rHhalignPara.pcaV;  }
+  if (rHhalignPara.pcbV  >= 0.00){ par.pcbV  = rHhalignPara.pcbV;  }
+  if (rHhalignPara.pccV  >= 0.00){ par.pccV  = rHhalignPara.pccV;  }
+  if (rHhalignPara.pcwV  >= 0.00){ par.pcwV  = rHhalignPara.pcwV;  }
+  if (rHhalignPara.gapbV >= 0.00){ par.gapbV = rHhalignPara.gapbV; }
+  if (rHhalignPara.gapdV >= 0.00){ par.gapdV = rHhalignPara.gapdV; }
+  if (rHhalignPara.gapeV >= 0.00){ par.gapeV = rHhalignPara.gapeV; }
+  if (rHhalignPara.gapfV >= 0.00){ par.gapfV = rHhalignPara.gapfV; }
+  if (rHhalignPara.gapgV >= 0.00){ par.gapgV = rHhalignPara.gapgV; }
+  if (rHhalignPara.gaphV >= 0.00){ par.gaphV = rHhalignPara.gaphV; }
+  if (rHhalignPara.gapiV >= 0.00){ par.gapiV = rHhalignPara.gapiV; }
 
 } /* this is the end of SetRnaDefaults() */
 
-void SetDnaDefaults()
+void SetDnaDefaults(hhalign_para rHhalignPara)
 {
   par.pca=2.89f;                // default values for substitution matrix pseudocounts 
   par.pcb=1.17f;                // significant reduction of pcs by Neff_M starts around Neff_M-1=pcb
@@ -804,9 +848,32 @@ void SetDnaDefaults()
   par.gapiV=0.645;                // factor for increasing gap extension penalty for inserts }
 #endif
 
-#if PARAMETERSFROMFILE  /* read parameter file from home-dir */
-#include "hhutil-C-help.h"
-#endif /* read parameter file from home-dir */
+  //#if PARAMETERSFROMFILE  /* read parameter file from home-dir */
+  //#include "hhutil-C-help.h"
+  //#endif /* read parameter file from home-dir */
+  if (rHhalignPara.pca  >= 0.00){ par.pca  = rHhalignPara.pca;  }
+  if (rHhalignPara.pcb  >= 0.00){ par.pcb  = rHhalignPara.pcb;  }
+  if (rHhalignPara.pcc  >= 0.00){ par.pcc  = rHhalignPara.pcc;  }
+  if (rHhalignPara.pcw  >= 0.00){ par.pcw  = rHhalignPara.pcw;  }
+  if (rHhalignPara.gapb >= 0.00){ par.gapb = rHhalignPara.gapb; }
+  if (rHhalignPara.gapd >= 0.00){ par.gapd = rHhalignPara.gapd; }
+  if (rHhalignPara.gape >= 0.00){ par.gape = rHhalignPara.gape; }
+  if (rHhalignPara.gapf >= 0.00){ par.gapf = rHhalignPara.gapf; }
+  if (rHhalignPara.gapg >= 0.00){ par.gapg = rHhalignPara.gapg; }
+  if (rHhalignPara.gaph >= 0.00){ par.gaph = rHhalignPara.gaph; }
+  if (rHhalignPara.gapi >= 0.00){ par.gapi = rHhalignPara.gapi; }
+  if (rHhalignPara.pcaV  >= 0.00){ par.pcaV  = rHhalignPara.pcaV;  }
+  if (rHhalignPara.pcbV  >= 0.00){ par.pcbV  = rHhalignPara.pcbV;  }
+  if (rHhalignPara.pccV  >= 0.00){ par.pccV  = rHhalignPara.pccV;  }
+  if (rHhalignPara.pcwV  >= 0.00){ par.pcwV  = rHhalignPara.pcwV;  }
+  if (rHhalignPara.gapbV >= 0.00){ par.gapbV = rHhalignPara.gapbV; }
+  if (rHhalignPara.gapdV >= 0.00){ par.gapdV = rHhalignPara.gapdV; }
+  if (rHhalignPara.gapeV >= 0.00){ par.gapeV = rHhalignPara.gapeV; }
+  if (rHhalignPara.gapfV >= 0.00){ par.gapfV = rHhalignPara.gapfV; }
+  if (rHhalignPara.gapgV >= 0.00){ par.gapgV = rHhalignPara.gapgV; }
+  if (rHhalignPara.gaphV >= 0.00){ par.gaphV = rHhalignPara.gaphV; }
+  if (rHhalignPara.gapiV >= 0.00){ par.gapiV = rHhalignPara.gapiV; }
+
 
 } /* this is the end of SetDnaDefaults() */
 
