@@ -15,7 +15,7 @@
  ********************************************************************/
 
 /*
- *  RCS $Id: mymain.c 274 2012-04-24 23:28:24Z dave $
+ *  RCS $Id: mymain.c 290 2013-09-20 15:18:12Z fabian $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -58,7 +58,13 @@ typedef struct {
     char *pcProfile1Infile ;
     /** profile 2: pre-aligned sequence input. not directly used by Align() */
     char *pcProfile2Infile;        
-    
+    /** profiles that contain no gaps are rejected, force them */
+    bool bIsProfile; 
+    /** up to version 1.1.1 Kimura distance was default, change default, make Kimura optional */
+    /*bool bUseKimura; */
+    /** distance matrix output is default, allow %-ID output */
+    bool bPercentID; 
+
     /** Input limitations
      */
     /** maximum allowed number of input sequences */
@@ -75,11 +81,21 @@ typedef struct {
     /** force overwriting of files */
     bool bForceFileOverwrite;
 
+    /* line wrapping, FS, r274 -> */
+    int iWrap;
+    /* residue number in Clustal format, FS, r274 -> */
+    bool bResno;
+    /* output order, FS, r274 -> */
+    int iOutputOrder; 
+
     /* multithreading
      */
     /** number of threads */
     int iThreads;
 
+    /* pseudo-count file
+     */
+    char *pcPseudoFile; 
     /* logging 
      */
     char *pcLogFile;
@@ -121,13 +137,19 @@ SetDefaultUserOpts(cmdline_opts_t *opts)
     opts->bDealignInputSeqs = FALSE;        
     opts->pcProfile1Infile = NULL;
     opts->pcProfile2Infile = NULL;
-	
+    opts->bIsProfile = FALSE;
+    opts->aln_opts.bUseKimura = FALSE;
+	opts->aln_opts.bPercID = FALSE;
+
     opts->iMaxNumSeq = INT_MAX;
     opts->iMaxSeqLen = INT_MAX;
     
     opts->pcAlnOutfile = NULL;
     opts->iAlnOutFormat =  MSAFILE_A2M;
     opts->bForceFileOverwrite = FALSE;
+    opts->iWrap = 60;
+    opts->bResno = FALSE;
+    opts->iOutputOrder = INPUT_ORDER;
 
 #ifdef HAVE_OPENMP
     /* defaults to # of CPUs */
@@ -136,11 +158,12 @@ SetDefaultUserOpts(cmdline_opts_t *opts)
     opts->iThreads = 1;
 #endif
     
+    opts->pcPseudoFile = NULL;
     opts->pcLogFile = NULL;
 
     SetDefaultAlnOpts(& opts->aln_opts);
 }
-/* end of SetDefaultAlnOpts() */
+/* end of SetDefaultUserOpts() */
 
 
 
@@ -163,13 +186,22 @@ PrintUserOpts(FILE *prFile, cmdline_opts_t *opts) {
             NULL != opts->pcProfile1Infile? opts->pcProfile1Infile: "(null)");
     fprintf(prFile, "option: profile2 = %s\n",
             NULL != opts->pcProfile2Infile? opts->pcProfile2Infile: "(null)");
+    /*fprintf(prFile, "option: percent-id = %d\n", opts->aln_opts.bPercID);*/
+    fprintf(prFile, "option: is-profile = %d\n", opts->bIsProfile);
+    /*fprintf(prFile, "option: use-kimura = %d\n", opts->aln_opts.bUseKimura);*/
+
     fprintf(prFile, "option: max-num-seq = %d\n", opts->iMaxNumSeq);
     fprintf(prFile, "option: max-seq-len = %d\n", opts->iMaxSeqLen);
     fprintf(prFile, "option: aln-out-file = %s\n", 
             NULL != opts->pcAlnOutfile? opts->pcAlnOutfile: "(null)");
     fprintf(prFile, "option: aln-out-format = %s\n", SeqfileFormat2String(opts->iAlnOutFormat));
     fprintf(prFile, "option: force-file-overwrite = %d\n", opts->bForceFileOverwrite);
+    fprintf(prFile, "option: line wrap = %d\n", opts->iWrap);
+    fprintf(prFile, "option: print residue numbers = %d\n", opts->bResno);
+    fprintf(prFile, "option: order alignment like input/tree = %d\n", opts->iOutputOrder);
+
     fprintf(prFile, "option: threads = %d\n", opts->iThreads);
+    fprintf(prFile, "option: PseudoFile = %s\n", opts->pcPseudoFile);
     fprintf(prFile, "option: logFile = %s\n", opts->pcLogFile);
 }
 /* end of PrintUserOpts */
@@ -204,6 +236,9 @@ FreeUserOpts(cmdline_opts_t *user_opts)
     if (NULL != user_opts->pcLogFile) {
         CKFREE(user_opts->pcLogFile);
     }
+    if (NULL != user_opts->pcPseudoFile) {
+        CKFREE(user_opts->pcPseudoFile);
+    }
 
     FreeAlnOpts(& user_opts->aln_opts);
 
@@ -249,10 +284,36 @@ UserOptsLogicCheck(cmdline_opts_t *opts)
               "You requested alignment output to stdout and verbose logging.",
               " Alignment and log messages would get mixed up.");
     }
-    /* distance matrix output impossible in mbed mode, only have clusters, FS, r254 ->  */
+    /* distance matrix output impossible in mBed mode, only have clusters, FS, r254 ->  */
+#if 1
+    /* allow distance matrix output if initial mBed but subsequent full matrix during iteration, FS, r274 -> */
+    if (NULL != opts->aln_opts.pcDistmatOutfile){
+        if ( (TRUE == opts->aln_opts.bUseMbed) && (opts->aln_opts.iNumIterations < 1) ){
+            Log(&rLog, LOG_FATAL, "Distance Matrix output not possible in mBed mode.");
+        }
+        if ( (TRUE == opts->aln_opts.bUseMbed) && (TRUE == opts->aln_opts.bUseMbedForIteration) ){
+            Log(&rLog, LOG_FATAL, "Distance Matrix output not possible in mBed mode.");
+        }
+        if ( (TRUE == opts->aln_opts.bUseMbed) && (opts->aln_opts.iNumIterations > 0) && 
+             (opts->aln_opts.iMaxGuidetreeIterations < 1) ){
+            Log(&rLog, LOG_FATAL, "Distance Matrix output not possible in mBed mode.");
+        }
+    }
+#else
     if ( (NULL != opts->aln_opts.pcDistmatOutfile) && (TRUE == opts->aln_opts.bUseMbed) ) {
         Log(&rLog, LOG_FATAL, "Distance Matrix output not possible in mBed mode.");
     }
+#endif
+
+    /* percentage identity cannot be printed in Kimura mode */
+    if ( (TRUE == opts->aln_opts.bUseKimura) && (TRUE == opts->aln_opts.bPercID) ){
+        Log(&rLog, LOG_FATAL, "Percentage Identity cannot be calculated if Kimura Distances are used.");
+    }
+
+    /* iteration destroys effect of pile-up */
+    if ( (TRUE == opts->aln_opts.bPileup) && (opts->aln_opts.iNumIterations > 0) ){
+        Log(&rLog, LOG_WARN, "Iteration destroys effect of pile-up.");
+    }
 
     AlnOptsLogicCheck(& opts->aln_opts);
 }
@@ -305,6 +366,9 @@ ParseCommandLine(cmdline_opts_t *user_opts, int argc, char **argv)
     struct arg_file *opt_profile2 = arg_file0(NULL, "profile2,p2",
                                               "<file>",
                                               "Pre-aligned multiple sequence file (aligned columns will be kept fix)");
+    struct arg_lit *opt_isprofile = arg_lit0(NULL, "is-profile",
+                                             "disable check if profile, force profile (default no)");
+
     struct arg_str *opt_seqtype = arg_str0("t", "seqtype",
                                            "{Protein, RNA, DNA}",
                                            "Force a sequence type (default: auto)");
@@ -313,7 +377,8 @@ ParseCommandLine(cmdline_opts_t *user_opts, int argc, char **argv)
     struct arg_str *opt_infmt = arg_str0(NULL, "infmt",
                                             "{a2m=fa[sta],clu[stal],msf,phy[lip],selex,st[ockholm],vie[nna]}",
                                             "Forced sequence input file format (default: auto)");
-
+    struct arg_lit *opt_resno = arg_lit0(NULL, "residuenumber,resno",
+                                         "in Clustal format print residue numbers (default no)");
     
     struct arg_rem  *rem_guidetree  = arg_rem(NULL, "\nClustering:");
     struct arg_str *opt_pairdist = arg_str0("p", "pairdist",
@@ -337,12 +402,28 @@ ParseCommandLine(cmdline_opts_t *user_opts, int argc, char **argv)
        struct arg_lit *opt_mbed_iter = arg_lit0(NULL, "mbed-iter",
        "Use Mbed-like clustering also during iteration");
     */
-    /* Note: might be better to use arg_str (mbed=YES/NO) but I don't want to introduce an '=' into pipeline, FS, r250 -> */
+    struct arg_lit *opt_pileup = arg_lit0(NULL, "pileup", 
+                                          "Sequentially align sequences");
     struct arg_lit *opt_full = arg_lit0(NULL, "full",
                                         "Use full distance matrix for guide-tree calculation (might be slow; mBed is default)");
     struct arg_lit *opt_full_iter = arg_lit0(NULL, "full-iter",
                                         "Use full distance matrix for guide-tree calculation during iteration (might be slowish; mBed is default)");
 
+    struct arg_int *opt_clustersize = arg_int0(NULL, "cluster-size", "<n>", 
+                                               "soft maximum of sequences in sub-clusters"); /* FS, r274 -> */
+
+    struct arg_file *opt_clustfile = arg_file0(NULL, "clustering-out",
+                                               "<file>",
+                                               "Clustering output file"); /* FS, r274 -> */
+    struct arg_int *opt_trans = arg_int0(NULL, "trans", "<n>", "use transitivity (default: 0)");
+    struct arg_file *opt_posteriorfile = arg_file0(NULL, "posterior-out",
+                                               "<file>",
+                                               "Posterior probability output file"); /* FS, r288 -> */
+    struct arg_lit *opt_usekimura = arg_lit0(NULL, "use-kimura",
+                                             "use Kimura distance correction for aligned sequences (default no)");
+    struct arg_lit *opt_percentid = arg_lit0(NULL, "percent-id",
+                                             "convert distances into percent identities (default no)");
+
     struct arg_str *opt_clustering = arg_str0("c", "clustering",
                                               "{UPGMA}",
                                               "Clustering method for guide tree");
@@ -355,6 +436,11 @@ ParseCommandLine(cmdline_opts_t *user_opts, int argc, char **argv)
     struct arg_str *opt_outfmt = arg_str0(NULL, "outfmt",
                                             "{a2m=fa[sta],clu[stal],msf,phy[lip],selex,st[ockholm],vie[nna]}",
                                             "MSA output file format (default: fasta)");
+    struct arg_int *opt_wrap = arg_int0(NULL, "wrap", "<n>", 
+                                        "number of residues before line-wrap in output");
+    struct arg_str *opt_output_order = arg_str0(NULL, "output-order",
+                                                "{input-order,tree-order}",
+                                                "MSA output order like in input/guide-tree");
 
     
     struct arg_rem *rem_iteration  = arg_rem(NULL, "\nIteration:");
@@ -362,7 +448,7 @@ ParseCommandLine(cmdline_opts_t *user_opts, int argc, char **argv)
                                                   /* FIXME "{<n>,auto}", "Number of combined guide-tree/HMM iterations"); */
                                                   "<n>", "Number of (combined guide-tree/HMM) iterations");
     struct arg_int *opt_max_guidetree_iterations = arg_int0(NULL, "max-guidetree-iterations",
-                                                            "<n>", "Maximum number guidetree iterations");
+                                                            "<n>", "Maximum number of guidetree iterations");
     struct arg_int *opt_max_hmm_iterations = arg_int0(NULL, "max-hmm-iterations",
                                                       "<n>", "Maximum number of HMM iterations");
 
@@ -380,6 +466,8 @@ ParseCommandLine(cmdline_opts_t *user_opts, int argc, char **argv)
                                          "Set options automatically (might overwrite some of your options)");
     struct arg_int *opt_threads = arg_int0(NULL, "threads", "<n>", 
                                               "Number of processors to use");
+    struct arg_file *opt_pseudo = arg_file0(NULL, "pseudo", "<file>",
+                                            "Input file for pseudo-count parameters");
     struct arg_file *opt_logfile = arg_file0("l", "log",
                                              "<file>",
                                              "Log all non-essential output to this file");
@@ -397,7 +485,6 @@ ParseCommandLine(cmdline_opts_t *user_opts, int argc, char **argv)
     struct arg_int *opt_macram = arg_int0(NULL, "MAC-RAM", "<n>", /* keep this quiet for the moment, FS r240 -> */
                                           NULL/*"maximum amount of RAM to use for MAC algorithm (in MB)"*/);
 
-
     struct arg_end *opt_end = arg_end(10); /* maximum number of errors
                                             * to store */
 
@@ -407,6 +494,7 @@ ParseCommandLine(cmdline_opts_t *user_opts, int argc, char **argv)
                         opt_dealign,
                         opt_profile1,
                         opt_profile2,
+                        opt_isprofile, /* FS, r282 ->*/
                         opt_seqtype,
                         /* opt_force_protein, */
                         opt_infmt,
@@ -419,8 +507,15 @@ ParseCommandLine(cmdline_opts_t *user_opts, int argc, char **argv)
                         opt_distmat_out,
                         opt_guidetree_in,
                         opt_guidetree_out,
+                        opt_pileup, /* FS, r288 -> */
                         opt_full, /* FS, r250 -> */
                         opt_full_iter, /* FS, r250 -> */
+                        opt_clustersize, /* FS, r274 -> */
+                        opt_clustfile, /* FS, r274 -> */
+                        opt_trans, /* FS, r290 -> */
+                        opt_posteriorfile, /* FS, r288 -> */
+                        opt_usekimura, /* FS, r282 ->*/
+                        opt_percentid, /* FS, r282 ->*/
 #if 0
                         /* no other options then default available */
                         opt_clustering,
@@ -428,6 +523,9 @@ ParseCommandLine(cmdline_opts_t *user_opts, int argc, char **argv)
                         rem_aln_output,
                         opt_outfile,
                         opt_outfmt,
+                        opt_resno,  /* FS, 274 -> */
+                        opt_wrap, /* FS, 274 -> */
+                        opt_output_order, /* FS, 274 -> */
 
                         rem_iteration,
                         opt_num_iterations,
@@ -441,6 +539,7 @@ ParseCommandLine(cmdline_opts_t *user_opts, int argc, char **argv)
                         rem_misc,
                         opt_autooptions,
                         opt_threads,
+                        opt_pseudo,
                         opt_logfile,
                         opt_help,
                         opt_verbose,
@@ -559,10 +658,13 @@ ParseCommandLine(cmdline_opts_t *user_opts, int argc, char **argv)
         user_opts->aln_opts.bUseMbedForIteration = FALSE;
     }
 
+    if (opt_pileup->count){
+        user_opts->aln_opts.bPileup = TRUE;
+    }
+
+
     user_opts->bForceFileOverwrite = opt_force->count;
 
-
-
     /* log-file
      */
     if (opt_logfile->count > 0) {
@@ -582,6 +684,16 @@ ParseCommandLine(cmdline_opts_t *user_opts, int argc, char **argv)
     }
 
 
+    /* pseudo-count-file
+     */
+    if (opt_pseudo->count > 0) {
+        user_opts->pcPseudoFile = CkStrdup(opt_pseudo->filename[0]);
+        if (! FileExists(user_opts->pcPseudoFile)) {
+            Log(&rLog, LOG_FATAL, "File '%s' does not exist.", user_opts->pcPseudoFile);
+        }
+    }
+
+
     /* normal sequence input (no profile)
      */
     if (opt_seqin->count > 0) {
@@ -644,6 +756,16 @@ ParseCommandLine(cmdline_opts_t *user_opts, int argc, char **argv)
         }
     }
     
+    if (opt_isprofile->count){
+        user_opts->bIsProfile = TRUE; 
+    }
+    if (opt_usekimura->count){
+        user_opts->aln_opts.bUseKimura = TRUE;
+    }
+    if (opt_percentid->count){
+        user_opts->aln_opts.bPercID = TRUE;
+    }
+
     
     /* HMM input
      */
@@ -713,7 +835,30 @@ ParseCommandLine(cmdline_opts_t *user_opts, int argc, char **argv)
         }
     }
 
-    
+    if (opt_clustersize->count > 0){ /* FS, r274 -> */
+        if (opt_clustersize->ival[0] > 0){
+            user_opts->aln_opts.iClustersizes = opt_clustersize->ival[0];
+        }
+    }
+
+    if (opt_clustfile->count > 0){ /* FS, r274 -> */
+        user_opts->aln_opts.pcClustfile = CkStrdup(opt_clustfile->filename[0]);
+        /*if (! FileExists(user_opts->aln_opts.pcClustfile)) {
+            Log(&rLog, LOG_FATAL, "File '%s' does not exist.", user_opts->aln_opts.pcClustfile);
+            }*/
+    }
+
+    if (opt_trans->count > 0){ /* FS, r290 -> */
+        user_opts->aln_opts.iTransitivity = opt_trans->ival[0];
+    }
+    if (opt_posteriorfile->count > 0){ /* FS, r288 -> */
+        user_opts->aln_opts.pcPosteriorfile = CkStrdup(opt_posteriorfile->filename[0]);
+        /*if (! FileExists(user_opts->aln_opts.pcPosteriorfile)) {
+            Log(&rLog, LOG_FATAL, "File '%s' does not exist.", user_opts->aln_opts.pcPosteriorfile);
+            }*/
+    }
+
+
     /* Guidetree input
      */
     if (opt_guidetree_in->count > 0) {
@@ -834,6 +979,24 @@ ParseCommandLine(cmdline_opts_t *user_opts, int argc, char **argv)
         user_opts->aln_opts.rHhalignPara.iMacRamMB = opt_macram->ival[0];
     }
 
+    /* Number of residues in output before line-wrap
+     */
+    if (opt_wrap->count > 0) { /* FS, r274 -> */
+        user_opts->iWrap = opt_wrap->ival[0];
+    }
+
+    user_opts->bResno = opt_resno->count;
+
+    /* output-order
+     * like input (INPUT_ORDER = 0) or tree (TREE_ORDER = 1)
+     * if output-order format not valid use INPUT_ORDER
+     */
+    if (opt_output_order->count > 0){
+        user_opts->iOutputOrder = (0 == strcmp(opt_output_order->sval[0], "input-order")) ? INPUT_ORDER : 
+            (0 == strcmp(opt_output_order->sval[0], "tree-order")) ? TREE_ORDER : INPUT_ORDER;
+    }
+
+
     arg_freetable(argtable,sizeof(argtable)/sizeof(argtable[0]));
 
     UserOptsLogicCheck(user_opts);
@@ -856,7 +1019,7 @@ MyMain(int argc, char **argv)
     mseq_t *prMSeq = NULL;
     mseq_t *prMSeqProfile1 = NULL;
     mseq_t *prMSeqProfile2 = NULL;
-    cmdline_opts_t cmdline_opts;
+    cmdline_opts_t cmdline_opts = {0};
 
     /* Must happen first: setup logger */
     LogDefaultSetup(&rLog);
@@ -881,6 +1044,11 @@ MyMain(int argc, char **argv)
         PrintUserOpts(LogGetFP(&rLog, LOG_INFO), & cmdline_opts);
         PrintAlnOpts(LogGetFP(&rLog, LOG_INFO), & (cmdline_opts.aln_opts));
     }
+#if 1
+    if (NULL != cmdline_opts.pcPseudoFile){
+        ReadPseudoCountParams(&cmdline_opts.aln_opts.rHhalignPara, cmdline_opts.pcPseudoFile);
+    }
+#endif
 
     /* Read sequence file
      *
@@ -889,6 +1057,7 @@ MyMain(int argc, char **argv)
         NewMSeq(&prMSeq);
         if (ReadSequences(prMSeq, cmdline_opts.pcSeqInfile,
                           cmdline_opts.iSeqType, cmdline_opts.iSeqInFormat,
+                          cmdline_opts.bIsProfile, cmdline_opts.bDealignInputSeqs, 
                           cmdline_opts.iMaxNumSeq, cmdline_opts.iMaxSeqLen)) {
             Log(&rLog, LOG_FATAL, "Reading sequence file '%s' failed", cmdline_opts.pcSeqInfile);
         }
@@ -910,11 +1079,19 @@ MyMain(int argc, char **argv)
         Log(&rLog, LOG_FATAL, "File '%s' contains %d sequence%s, nothing to align",
               cmdline_opts.pcSeqInfile, prMSeq->nseqs, 1==prMSeq->nseqs?"":"s");
     }
+    /* if there are fewer sequences than target size of clusters,
+     * then mBed is unnecessary, FS, r283-> */
+    if ( (prMSeq) && (prMSeq->nseqs <= cmdline_opts.aln_opts.iClustersizes) ){
+        cmdline_opts.aln_opts.bUseMbed = FALSE;
+        cmdline_opts.aln_opts.bUseMbedForIteration = FALSE;
+        Log(&rLog, LOG_INFO, "not more sequences (%d) than cluster-size (%d), turn off mBed", 
+            prMSeq->nseqs, cmdline_opts.aln_opts.iClustersizes);
+    }
 
     /* Dealign if requested and neccessary
      */
     if (NULL != prMSeq) {
-        if (TRUE == prMSeq->aligned && cmdline_opts.bDealignInputSeqs) {
+        if (/*TRUE == prMSeq->aligned &&*/ cmdline_opts.bDealignInputSeqs) {
             Log(&rLog, LOG_INFO, "Dealigning already aligned input sequences as requested.");
             DealignMSeq(prMSeq);
         }
@@ -928,6 +1105,7 @@ MyMain(int argc, char **argv)
         NewMSeq(&prMSeqProfile1);
         if (ReadSequences(prMSeqProfile1, cmdline_opts.pcProfile1Infile,
                           cmdline_opts.iSeqType,  cmdline_opts.iSeqInFormat,
+                          cmdline_opts.bIsProfile, cmdline_opts.bDealignInputSeqs, 
                           cmdline_opts.iMaxNumSeq, cmdline_opts.iMaxSeqLen)) {
             Log(&rLog, LOG_FATAL, "Reading sequences from profile file '%s' failed",
                   cmdline_opts.pcProfile1Infile);
@@ -953,6 +1131,7 @@ MyMain(int argc, char **argv)
         NewMSeq(&prMSeqProfile2);
         if (ReadSequences(prMSeqProfile2, cmdline_opts.pcProfile2Infile,
                           cmdline_opts.iSeqType,  cmdline_opts.iSeqInFormat,
+                          cmdline_opts.bIsProfile, cmdline_opts.bDealignInputSeqs, 
                           cmdline_opts.iMaxNumSeq, cmdline_opts.iMaxSeqLen)) {
             Log(&rLog, LOG_FATAL, "Reading sequences from profile file '%s' failed",
                   cmdline_opts.pcProfile2Infile);
@@ -967,7 +1146,7 @@ MyMain(int argc, char **argv)
           Log(&rLog, LOG_FATAL, "'%s' contains only one sequence and can therefore not be used as a profile",
           cmdline_opts.pcProfile2Infile);
           }*/
-        if (FALSE == prMSeqProfile1->aligned) {
+        if (FALSE == prMSeqProfile2->aligned) {
             Log(&rLog, LOG_FATAL, "Sequences in '%s' are not aligned, i.e. this is not a profile",
                   cmdline_opts.pcProfile2Infile);
         }
@@ -981,14 +1160,30 @@ MyMain(int argc, char **argv)
      * (ii) profile profile alignment
      *
      */
+
     if (NULL != prMSeq) {
+
+        if (2 == prMSeq->nseqs){
+            /* if there are only 2 sequences then the order does not matter */
+            /* this is important, because for pair-wise alignment we don't do 
+             * tree indexing*, and trying to use tree-indexing during output 
+             * will cause a segmentation fault */
+            cmdline_opts.iOutputOrder = INPUT_ORDER;
+        }
+        if (TREE_ORDER == cmdline_opts.iOutputOrder){
+            /* this is a crutch, if tree_order==NULL use input order, 
+             * otherwise use guide-tree-order */
+            prMSeq->tree_order = (int *)CKMALLOC(prMSeq->nseqs * sizeof(int));
+        }
         if (Align(prMSeq, prMSeqProfile1, & cmdline_opts.aln_opts)) {
             Log(&rLog, LOG_FATAL, "An error occured during the alignment");
         }
 
-        if (WriteAlignment(prMSeq, cmdline_opts.pcAlnOutfile, 
-                           cmdline_opts.iAlnOutFormat)) {
-            Log(&rLog, LOG_FATAL, "Could not save alignment to %s", cmdline_opts.pcAlnOutfile);
+        if (cmdline_opts.aln_opts.iMaxHMMIterations >= 0){
+            if (WriteAlignment(prMSeq, cmdline_opts.pcAlnOutfile, 
+                               cmdline_opts.iAlnOutFormat, cmdline_opts.iWrap, cmdline_opts.bResno)) {
+                Log(&rLog, LOG_FATAL, "Could not save alignment to %s", cmdline_opts.pcAlnOutfile);
+            }
         }
 #if 0
         {
@@ -1005,7 +1200,7 @@ MyMain(int argc, char **argv)
             Log(&rLog, LOG_FATAL, "An error occured during the alignment");
         }
         if (WriteAlignment(prMSeqProfile1, cmdline_opts.pcAlnOutfile, 
-                           cmdline_opts.iAlnOutFormat)) {
+                           cmdline_opts.iAlnOutFormat, cmdline_opts.iWrap, cmdline_opts.bResno)) {
             Log(&rLog, LOG_FATAL, "Could not save alignment to %s", cmdline_opts.pcAlnOutfile);
         }
     }
