@@ -14,9 +14,12 @@
  * formatted files. Derivative of msf.c; MSF is a pretty
  * generic interleaved format.   
  * 
- * RCS $Id: clustal.c 277 2013-05-16 15:42:49Z fabian $ (Original squid RCS Id: clustal.c,v 1.1 1999/07/15 22:26:53 eddy Exp)
+ * RCS $Id: clustal.c 291 2014-02-27 18:20:54Z fabian $ (Original squid RCS Id: clustal.c,v 1.1 1999/07/15 22:26:53 eddy Exp)
  */
 
+#include <stdbool.h>
+#include <unistd.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -174,6 +177,13 @@ ReadClustal(MSAFILE *afp)
   return msa;
 }
 
+size_t utf8len(char *s)
+{
+  size_t len = 0;
+  for (; *s; ++s) if ((*s & 0xC0) != 0x80) ++len;
+  return len;
+}
+
 
 /* Function: WriteClustal()
  * Date:     SRE, Sun Jun  6 18:12:47 1999 [bus from Madison, worm mtg 1999]
@@ -187,7 +197,7 @@ ReadClustal(MSAFILE *afp)
  */
 void
 #ifdef CLUSTALO
-WriteClustal(FILE *fp, MSA *msa, int iWrap, int bResno)
+WriteClustal(FILE *fp, MSA *msa, int iWrap, int bResno, int iSeqType)
 #else
 WriteClustal(FILE *fp, MSA *msa)
 #endif
@@ -210,7 +220,7 @@ WriteClustal(FILE *fp, MSA *msa)
   int bail;
   int strong_bins[9];
   int weak_bins[11];
-  int cons;
+  /*int cons;*/
   int bin;
 
 #ifdef CLUSTALO
@@ -242,9 +252,13 @@ WriteClustal(FILE *fp, MSA *msa)
 
   /* calculate max namelen used */
   namelen = 0;
-  for (idx = 0; idx < msa->nseq; idx++)
-    if ((len = strlen(msa->sqname[idx])) > namelen) 
-      namelen = len;
+  for (idx = 0; idx < msa->nseq; idx++){
+    /*if ((len = strlen(msa->sqname[idx])) > namelen) */ /* strlen() gives problems for unicode, FS, -> 290 */
+    if ((len = utf8len(msa->sqname[idx])) > namelen)
+      {
+	namelen = len; 
+      }
+  }
 
 #ifdef CLUSTALO
   fprintf(fp, "CLUSTAL O(%s) multiple sequence alignment\n", PACKAGE_VERSION);
@@ -275,10 +289,14 @@ WriteClustal(FILE *fp, MSA *msa)
 		  piResCnt[idx]++;
 		}
 	      }
-	      fprintf(fp, "%-*s %s\t%d\n", namelen+5, msa->sqname[idx], buf, piResCnt[idx]);
+	      /* printf("%*s") gives problems for unicode, FS, -> 290 */
+	      /*fprintf(fp, "%-*s\t%s\t%d\n", namelen+5, msa->sqname[idx], buf, piResCnt[idx]);*/
+	      fprintf(fp, "%s%*s %s\t%d\n", msa->sqname[idx], namelen+5-utf8len(msa->sqname[idx]), "", buf, piResCnt[idx]);
 	    }
 	    else {
-	      fprintf(fp, "%-*s %s\n", namelen+5, msa->sqname[idx], buf);
+	      /* printf("%*s") gives problems for unicode, FS, -> 290 */
+	      /*fprintf(fp, "%-*s\t%s\n", namelen+5, msa->sqname[idx], buf);*/
+	      fprintf(fp, "%s%*s %s\n", msa->sqname[idx], namelen+5-utf8len(msa->sqname[idx]), "", buf);
 	    }
 #else
 	    fprintf(fp, "%*s %s\n", namelen, msa->sqname[idx], buf);
@@ -298,7 +316,7 @@ WriteClustal(FILE *fp, MSA *msa)
           bail = 0;
           for (idx = 1; idx < msa->nseq; idx++)
           {
-            if(msa->aseq[idx][subpos] != first)
+            if(toupper(msa->aseq[idx][subpos]) != toupper(first)) /* toupper makes consensus case-insensitive, FS, r290 -> */
             {
               bail = 1;
               break;
@@ -312,9 +330,9 @@ WriteClustal(FILE *fp, MSA *msa)
             for(bin = 0; bin < 9; bin++)
               strong_bins[bin] = 0; /* clear the bins */
 
-            for(idx = 0; idx < msa->nseq; idx++)
-            {
-              switch(msa->aseq[idx][subpos])
+            for (idx = 0; (iSeqType == kAmino) && (idx < msa->nseq); idx++)
+            { /* do this only for amino acids, no strong/weak consensus for nucleotide, FS, r290 -> */
+              switch(toupper(msa->aseq[idx][subpos])) /* toupper makes consensus case-insensitive, FS, r290 -> */
               {
                 case 'S': strong_bins[0]++; break;
                 case 'T': strong_bins[0]++; break;
@@ -350,9 +368,9 @@ WriteClustal(FILE *fp, MSA *msa)
               for(bin = 0; bin < 11; bin++)
                 weak_bins[bin] = 0; /* clear the bins */
 
-              for(idx = 0; idx < msa->nseq; idx++)
-              {
-                switch(msa->aseq[idx][subpos])
+              for(idx = 0; (iSeqType == kAmino) && (idx < msa->nseq); idx++)
+	      { /* do this only for amino acids, no strong/weak consensus for nucleotide, FS, r290 -> */
+                switch(toupper(msa->aseq[idx][subpos])) /* toupper makes consensus case-insensitive, FS, r290 -> */
                 {
                   case 'C': weak_bins[0]++; break;
                   case 'S': weak_bins[0]++; weak_bins[2]++; weak_bins[3]++; weak_bins[4]++; weak_bins[5]++; weak_bins[6]++; break;
@@ -392,7 +410,7 @@ WriteClustal(FILE *fp, MSA *msa)
 #endif
     }
 
-#ifdef CLUSTAL)
+#ifdef CLUSTAL
   free(piResCnt); piResCnt = NULL;
 #endif
 
