@@ -13,7 +13,7 @@
  * SQUID's interface for multiple sequence alignment
  * manipulation: access to the MSA object.
  * 
- * RCS $Id: msa.c 277 2013-05-16 15:42:49Z fabian $ (Original squid RCS Id: msa.c,v 1.18 2002/10/12 04:40:35 eddy Exp)
+ * RCS $Id: msa.c 291 2014-02-27 18:20:54Z fabian $ (Original squid RCS Id: msa.c,v 1.18 2002/10/12 04:40:35 eddy Exp)
  */
 
 #include <stdio.h>
@@ -958,18 +958,18 @@ MSAFileGetLine(MSAFILE *afp)
 }
 
 void 
-MSAFileWrite(FILE *fp, MSA *msa, int outfmt, int do_oneline, int iWrap, int bResno)
+MSAFileWrite(FILE *fp, MSA *msa, int outfmt, int do_oneline, int iWrap, int bResno, int iSeqType)
 {
   switch (outfmt) {
 #ifdef CLUSTALO
-    //case MSAFILE_A2M:       WriteA2M(stdout, msa, 0);                 break;
-    //case MSAFILE_VIENNA:    WriteA2M(stdout, msa, 1);                 break;
+    /*case MSAFILE_A2M:       WriteA2M(stdout, msa, 0);                 break;*/
+    /*case MSAFILE_VIENNA:    WriteA2M(stdout, msa, 1);                 break;*/
   case MSAFILE_A2M:       WriteA2M(stdout, msa, iWrap);                 break;
 #ifndef INT_MAX
 #define INT_MAX 2147483647
 #endif
   case MSAFILE_VIENNA:    WriteA2M(stdout, msa, INT_MAX);               break;
-  case MSAFILE_CLUSTAL:   WriteClustal(stdout, msa, iWrap, bResno); break;
+  case MSAFILE_CLUSTAL:   WriteClustal(stdout, msa, iWrap, bResno, iSeqType); break;
 #else
   case MSAFILE_A2M:       WriteA2M(stdout, msa);     break;
   case MSAFILE_CLUSTAL:   WriteClustal(stdout, msa); break;
