@@ -16,7 +16,7 @@
  * Header file for SQUID's multiple sequence alignment 
  * manipulation code.
  * 
- * RCS $Id: msa.h 277 2013-05-16 15:42:49Z fabian $ (Original squid RCS Id: msa.h,v 1.12 2002/10/12 04:40:35 eddy Exp)
+ * RCS $Id: msa.h 291 2014-02-27 18:20:54Z fabian $ (Original squid RCS Id: msa.h,v 1.12 2002/10/12 04:40:35 eddy Exp)
  */
 
 #include <stdio.h>		/* FILE support */
@@ -226,7 +226,7 @@ extern MSA     *MSAFileRead(MSAFILE *afp);
 extern void     MSAFileClose(MSAFILE *afp);
 extern void     MSAFree(MSA *msa);
 #ifdef CLUSTALO
-extern void     MSAFileWrite(FILE *fp, MSA *msa, int outfmt, int do_oneline, int iWrap, int bResno);
+extern void     MSAFileWrite(FILE *fp, MSA *msa, int outfmt, int do_oneline, int iWrap, int bResno, int iSeqtype);
 #else
 extern void     MSAFileWrite(FILE *fp, MSA *msa, int outfmt, int do_oneline);
 #endif
@@ -268,7 +268,7 @@ extern float MSAAverageSequenceLength(MSA *msa);
  */
 extern MSA  *ReadA2M(MSAFILE *afp);
 #ifdef CLUSTALO
-//extern void  WriteA2M(FILE *fp, MSA *msa, int vienna);
+/*extern void  WriteA2M(FILE *fp, MSA *msa, int vienna);*/
 extern void  WriteA2M(FILE *fp, MSA *msa, int iWrap);
 #else
 extern void  WriteA2M(FILE *fp, MSA *msa);
@@ -277,7 +277,7 @@ extern void  WriteA2M(FILE *fp, MSA *msa);
  */
 extern MSA  *ReadClustal(MSAFILE *afp);
 #ifdef CLUSTALO
-extern void  WriteClustal(FILE *fp, MSA *msa, int iWrap, int bResno);
+extern void  WriteClustal(FILE *fp, MSA *msa, int iWrap, int bResno, int iSeqType);
 #else
 extern void  WriteClustal(FILE *fp, MSA *msa);
 #endif
