@@ -865,6 +865,18 @@ endPearson(char *s, int *addend)
   return(*s == '>');
 }
 
+#ifdef CLUSTALO
+static int
+only_whitespace(const char *s) {
+  while (*s != '\0') {
+    if (!isspace((unsigned char)*s))
+      return 0;
+    s++;
+  }
+  return 1;
+}
+#endif
+
 static void 
 readPearson(struct ReadSeqVars *V)
 {
@@ -873,7 +885,7 @@ readPearson(struct ReadSeqVars *V)
   if (V->ssimode >= 0) V->r_off = V->ssioffset;
 
 #ifdef CLUSTALO
-  while (*V->buf == '\n') {
+  while (only_whitespace(V->buf)) {
     SeqfileGetLine(V);
   }
 
@@ -881,12 +893,19 @@ readPearson(struct ReadSeqVars *V)
 #else
   if (*V->buf != '>') 
 #endif
+#ifdef CLUSTALO
+    Die("\
+File %s does not appear to be in FASTA format at line %d.\n\
+You may want to specify the file format on the command line.\n\
+Usually this is done with an option --infmt <fmt>.\n", 
+	V->fname, V->linenumber);
+#else
     Die("\
 File %s does not appear to be in FASTA format at line %d.\n\
 You may want to specify the file format on the command line.\n\
 Usually this is done with an option --informat <fmt>.\n", 
 	V->fname, V->linenumber);
-
+#endif
   if ((sptr = strtok(V->buf+1, "\n\t ")) != NULL)
     SetSeqinfoString(V->sqinfo, sptr, SQINFO_NAME);
   if ((sptr = strtok(NULL, "\n")) != NULL)
